from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch

from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader
from mteb.types import PromptType

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput

logger = logging.getLogger(__name__)

KALM_EMBEDDING_CITATION = """@article{hu2025kalm,
  title={KaLM-Embedding: Superior Training Data Brings A Stronger Embedding Model},
  author={Hu, Xinshuo and Shan, Zifei and Zhao, Xinping and Sun, Zetian and Liu, Zhenyu and Li, Dongfang and Ye, Shaolin and Wei, Xinyuan and Chen, Qian and Hu, Baotian and others},
  journal={arXiv preprint arXiv:2501.01028},
  year={2025}
}"""


class KALMWrapper(InstructSentenceTransformerModel):
    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        _inputs = [text for batch in inputs for text in batch["text"]]

        instruction = self.get_task_instruction(task_metadata, prompt_type)

        # to passage prompts won't be applied to passages
        if (
            not self.apply_instruction_to_passages
            and prompt_type == PromptType.document
        ):
            instruction = None
            logger.info(
                f"No instruction used, because prompt type = {prompt_type.document}"
            )

        if task_metadata.type in ["STS", "PairClassification", "Summarization"]:
            logger.info(
                f"No instruction used, because task type = {task_metadata.type}"
            )
            instruction = None

        if instruction:
            logger.info(
                f"Using instruction: '{instruction}' for task: '{task_metadata.name}'"
            )

        embeddings = self.model.encode(
            _inputs,
            prompt=instruction,
            **kwargs,
        )

        if isinstance(embeddings, torch.Tensor):
            # sometimes in kwargs can be return_tensors=True
            embeddings = embeddings.cpu().detach().float().numpy()
        return embeddings


kalm_training_data = {
    # from technical report
    # not in MTEB:
    # ExpertQA
    # MEDI2BGE
    # OpenOrca
    # PAQ
    # PubMedQA
    # SearchQA
    # arxiv_qa
    # rag-dataset-12000
    # CC-News
    # SQuAD 2.0
    # TriviaQA
    # WebGPT Comparisons
    # MultiNLI
    # NLLB
    # WikiAnswers
    # SimCSE NLI
    # SNLI
    # Aya Dataset
    # eli5
    # ----
    # in MTEB:
    "CodeFeedbackMT",
    "CodeFeedbackST",
    "ArxivClusteringP2P",
    "ArxivClusteringS2S",
    "ArxivClusteringP2P.v2",
    "TRECCOVID",
    "DBPedia",
    "ESCIReranking",
    "FEVER",
    "FiQA2018",
    "FEVERHardNegatives",
    "NanoFEVERRetrieval",
    "FEVER-NL",  # translation not trained on
    "FiQA2018-NL",  # translation not trained on
    "HotpotQA-PL",  # translation not trained on
    "HotpotQA-NL",  # translation not trained on
    "HotpotQAHardNegatives",
    "MultiLongDocRetrieval",
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",  # translation not trained on
    "mMARCO-NL",  # translation not trained on
    "MSMARCOv2",
    "NFCorpus",
    "SciFact",
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    "NQ-NL",  # translation not trained on
    "YahooAnswersTopicsClassification",
    "ContractNLIConfidentialityOfAgreementLegalBenchClassification",
    "ContractNLIExplicitIdentificationLegalBenchClassification",
    "ContractNLIInclusionOfVerballyConveyedInformationLegalBenchClassification",
    "ContractNLILimitedUseLegalBenchClassification",
    "ContractNLINoLicensingLegalBenchClassification",
    "ContractNLINoticeOnCompelledDisclosureLegalBenchClassification",
    "ContractNLIPermissibleAcquirementOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissibleCopyLegalBenchClassification",
    "ContractNLIPermissibleDevelopmentOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissiblePostAgreementPossessionLegalBenchClassification",
    "ContractNLIReturnOfConfidentialInformationLegalBenchClassification",
    "ContractNLISharingWithEmployeesLegalBenchClassification",
    "ContractNLISharingWithThirdPartiesLegalBenchClassification",
    "ContractNLISurvivalOfObligationsLegalBenchClassification",
    "QuoraRetrieval",
    "NanoQuoraRetrieval",
    "BiorxivClusteringP2P.v2",
    "BiorxivClusteringS2S.v2",
    "MedrxivClusteringP2P.v2",
    "MedrxivClusteringS2S.v2",
    "Banking77Classification",
    "AmazonPolarityClassification",
    "ImdbClassification",
    "EmotionClassification",
    "TweetSentimentExtractionClassification",
    "ToxicConversationsClassification",
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",
    "MrTidyRetrieval",
    "PawsXPairClassification",
    "AmazonReviewsClassification",
    "AmazonCounterfactualClassification",
    "MultilingualSentiment",
    "MassiveIntentClassification",
    "MassiveScenarioClassification",
    "MTOPDomainClassification",
    "MTOPIntentClassification",
}


kalm_v2_training_data = {
    "CodeFeedbackMT",
    "CodeFeedbackST",
    "ArxivClusteringP2P",
    "ArxivClusteringS2S",
    "ArxivClusteringP2P.v2",
    "TRECCOVID",
    "DBPedia",
    "ESCIReranking",
    "FEVER",
    "FiQA2018",
    "FEVERHardNegatives",
    "NanoFEVERRetrieval",
    "FEVER-NL",
    "FiQA2018-NL",
    "HotpotQA-PL",
    "HotpotQA-NL",
    "HotpotQAHardNegatives",
    "MultiLongDocRetrieval",
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",
    "mMARCO-NL",
    "MSMARCOv2",
    "NFCorpus",
    "SciFact",
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",
    "NQ-NL",
    "YahooAnswersTopicsClassification",
    "ContractNLIConfidentialityOfAgreementLegalBenchClassification",
    "ContractNLIExplicitIdentificationLegalBenchClassification",
    "ContractNLIInclusionOfVerballyConveyedInformationLegalBenchClassification",
    "ContractNLILimitedUseLegalBenchClassification",
    "ContractNLINoLicensingLegalBenchClassification",
    "ContractNLINoticeOnCompelledDisclosureLegalBenchClassification",
    "ContractNLIPermissibleAcquirementOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissibleCopyLegalBenchClassification",
    "ContractNLIPermissibleDevelopmentOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissiblePostAgreementPossessionLegalBenchClassification",
    "ContractNLIReturnOfConfidentialInformationLegalBenchClassification",
    "ContractNLISharingWithEmployeesLegalBenchClassification",
    "ContractNLISharingWithThirdPartiesLegalBenchClassification",
    "ContractNLISurvivalOfObligationsLegalBenchClassification",
    "QuoraRetrieval",
    "NanoQuoraRetrieval",
    "BiorxivClusteringP2P.v2",
    "BiorxivClusteringS2S.v2",
    "MedrxivClusteringP2P.v2",
    "MedrxivClusteringS2S.v2",
    "Banking77Classification",
    "AmazonPolarityClassification",
    "ImdbClassification",
    "EmotionClassification",
    "TweetSentimentExtractionClassification",
    "ToxicConversationsClassification",
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",
    "MrTidyRetrieval",
    "PawsXPairClassification",
    "AmazonReviewsClassification",
    "AmazonCounterfactualClassification",
    "MultilingualSentiment",
    "MassiveIntentClassification",
    "MassiveScenarioClassification",
    "MTOPDomainClassification",
    "MTOPIntentClassification",
    "Reddit-Clustering",
    "Reddit-Clustering-P2P",
    "Stackexchange-Clustering",
    "Stackexchange-Clustering-P2P",
    "TwentyNewsgroups-Clustering",
    "ATEC",
    "BQ",
    "CQADupstack",
}


KaLM_Embedding_gemma_3_12b_training_data = [
    "BornholmBitextMining",
    "BibleNLPBitextMining",  # train[256:] to avoid the test set in MTEB
    "FinParaSTS",
    "SemRel24STS",
    "STSB",
    "GermanSTSBenchmark",
    "TERRa",
    "StatcanDialogueDatasetRetrieval",
    "SwissJudgementClassification",
    "GreekLegalCodeClassification",
    "CataloniaTweetClassification",
    "NordicLangClassification",
    "CyrillicTurkicLangClassification",
    "PoemSentimentClassification",
    "CzechProductReviewSentimentClassification",
    "DBpediaClassification",
    "IndicLangClassification",
    "NusaParagraphEmotionClassification",
    "OdiaNewsClassification",
    "ScalaClassification",
    "ItaCaseholdClassification",
    "CSFDSKMovieReviewSentimentClassification",
    "ToxicConversationsClassification",
    "PAC",
    "SpartQA",
    "WinoGrande",
    "Quail",
    "HellaSwag",
    "AlphaNLI",
]
KaLM_Embedding_gemma_3_12b_training_data += kalm_training_data


KaLM_task_prompts = {
    "AmazonCounterfactualClassification": "Given an Amazon review, judge whether it is counterfactual.",
    "AmazonPolarityClassification": "Classifying Amazon reviews into positive or negative sentiment",
    "AmazonReviewsClassification": "Classifying the given Amazon review into its appropriate rating category",
    "Banking77Classification": "Given an online banking query, find the corresponding intents",
    "EmotionClassification": "Classifying the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise",
    "ImdbClassification": "Classifying the sentiment expressed in the given movie review text from the IMDB dataset",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios",
    "MTOPDomainClassification": "Classifying the intent domain of the given utterance in task-oriented conversation",
    "MTOPIntentClassification": "Classifying the intent of the given utterance in task-oriented conversation",
    "ToxicConversationsClassification": "Classifying the given comments as either toxic or not toxic",
    "TweetSentimentExtractionClassification": "Classifying the sentiment of a given tweet as either positive, negative, or neutral",
    "TNews": "Categorizing the given news title",
    "IFlyTek": "Given an App description text, find the appropriate fine-grained category",
    "MultilingualSentiment": "Classifying sentiment of the customer review into positive, neutral, or negative",
    "JDReview": "Classifying sentiment of the customer review for iPhone into positive or negative",
    "OnlineShopping": "Classifying sentiment of the customer review into positive or negative",
    "Waimai": "Classify the customer review from a food takeaway platform into positive or negative",
    "MasakhaNEWSClassification": "Classifying the category of french news.",
    "CBD": "Classifying the sentiment of polish tweet reviews",
    "PolEmo2.0-IN": "Classifying the sentiment of in-domain (medicine and hotels) online reviews",
    "PolEmo2.0-OUT": "Classifying the sentiment of out-of-domain (products and school) online reviews",
    "AllegroReviews": "Classifying the sentiment of reviews from e-commerce marketplace Allegro",
    "PAC": 'Classifying the sentence into one of the two types: "BEZPIECZNE_POSTANOWIENIE_UMOWNE" and "KLAUZULA_ABUZYWNA"',
    "GeoreviewClassification": "Classifying the sentiment of Russian reviews.",
    "HeadlineClassification": "Classifying the topic of Russian headlines.",
    "InappropriatenessClassification": "Detecting inappropriate messages on sensitive topics",
    "KinopoiskClassification": "Classifying the sentiment of Kinopoisk reviews.",
    "RuReviewsClassification": "Classifying the sentiment of Russian product reviews.",
    "RuSciBenchGRNTIClassification": "Classifying the topic of Russian scientific papers.",
    "RuSciBenchOECDClassification": "Classifying the topic of Russian scientific papers.",
    "CEDRClassification": "Classification of sentences by emotions.",
    "SensitiveTopicsClassification": "Detecting inappropriate messages on sensitive topics.",
    "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArxivClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BiorxivClusteringP2P": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringS2S": "Identify the main category of Biorxiv papers based on the titles",
    "MedrxivClusteringP2P": "Identify the main category of Medrxiv papers based on the titles and abstracts",
    "MedrxivClusteringS2S": "Identify the main category of Medrxiv papers based on the titles",
    "RedditClustering": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "RedditClusteringP2P": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "StackExchangeClustering": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "StackExchangeClusteringP2P": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "TwentyNewsgroupsClustering": "Identify the topic or theme of the given news articles",
    "CLSClusteringS2S": "Identify the main category of scholar papers based on the titles",
    "CLSClusteringP2P": "Identify the main category of scholar papers based on the titles and abstracts",
    "ThuNewsClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "ThuNewsClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    "AlloProfClusteringP2P": "Identify the main category of Allo Prof document based on the titles and descriptions",
    "AlloProfClusteringS2S": "Identify the main category of Allo Prof document based on the titles",
    "HALClusteringS2S": "Identify the main category of academic passage based on the titles and contents",
    "MasakhaNEWSClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    "MasakhaNEWSClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "MLSUMClusteringP2P": "Identify the topic or theme of the given articles based on the titles and contents",
    "MLSUMClusteringS2S": "Identify the topic or theme of the given articles based on the titles",
    "EightTagsClustering": "Identify of headlines from social media posts in Polish into 8 categories: film, history, food, medicine, motorization, work, sport and technology",
    "GeoreviewClusteringP2P": "Identify the topic or theme of the Russian reviews.",
    "RuSciBenchGRNTIClusteringP2P": "Identify the topic or theme of the Russian articles.",
    "RuSciBenchOECDClusteringP2P": "Identify the topic or theme of the Russian articles.",
}

KaLM_v2_task_prompts = {
    "AmazonCounterfactualClassification": "Given an Amazon review, judge whether it is counterfactual.",
    "AmazonPolarityClassification": "Classifying Amazon reviews into positive or negative sentiment",
    "AmazonReviewsClassification": "Classifying the given Amazon review into its appropriate rating category",
    "Banking77Classification": "Given an online banking query, find the corresponding intents",
    "EmotionClassification": "Classifying the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise",
    "ImdbClassification": "Classifying the sentiment expressed in the given movie review text from the IMDB dataset",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios",
    "MTOPDomainClassification": "Classifying the intent domain of the given utterance in task-oriented conversation",
    "MTOPIntentClassification": "Classifying the intent of the given utterance in task-oriented conversation",
    "ToxicConversationsClassification": "Classifying the given comments as either toxic or not toxic",
    "TweetSentimentExtractionClassification": "Classifying the sentiment of a given tweet as either positive, negative, or neutral",
    "TNews": "Categorizing the given news title",
    "IFlyTek": "Given an App description text, find the appropriate fine-grained category",
    "MultilingualSentiment": "Classifying sentiment of the customer review into positive, neutral, or negative",
    "JDReview": "Classifying sentiment of the customer review for iPhone into positive or negative",
    "OnlineShopping": "Classifying sentiment of the customer review into positive or negative",
    "Waimai": "Classify the customer review from a food takeaway platform into positive or negative",
    "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArxivClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BiorxivClusteringP2P": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringS2S": "Identify the main category of Biorxiv papers based on the titles",
    "MedrxivClusteringP2P": "Identify the main category of Medrxiv papers based on the titles and abstracts",
    "MedrxivClusteringS2S": "Identify the main category of Medrxiv papers based on the titles",
    "RedditClustering": "Identify the topic or theme of Reddit posts based on the titles",
    "RedditClusteringP2P": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "StackExchangeClustering": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClusteringP2P": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "TwentyNewsgroupsClustering": "Identify the topic or theme of the given news articles",
    "CLSClusteringS2S": "Identify the main category of scholar papers based on the titles",
    "CLSClusteringP2P": "Identify the main category of scholar papers based on the titles and abstracts",
    "ThuNewsClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "ThuNewsClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    "Cmnli-query": "Retrieve semantically similar text",
    "Cmnli-document": "Retrieve semantically similar text",
    "Ocnli-query": "Retrieve semantically similar text",
    "Ocnli-document": "Retrieve semantically similar text",
    "SprintDuplicateQuestions-query": "Retrieve semantically similar questions",
    "SprintDuplicateQuestions-document": "Retrieve semantically similar questions",
    "TwitterSemEval2015-query": "Retrieve semantically similar text",
    "TwitterSemEval2015-document": "Retrieve semantically similar text",
    "TwitterURLCorpus-query": "Retrieve semantically similar text",
    "TwitterURLCorpus-document": "Retrieve semantically similar text",
    "CMedQAv1-reranking": "Given a query, retrieve documents that answer the query",
    "CMedQAv2-reranking": "Given a query, retrieve documents that answer the query",
    "MMarcoReranking": "Given a query, retrieve documents that answer the query",
    "T2Reranking": "Given a query, retrieve documents that answer the query",
    "AskUbuntuDupQuestions-query": "Retrieve semantically similar questions",
    "AskUbuntuDupQuestions-document": "Retrieve semantically similar questions",
    "MindSmallReranking": "Given a query, retrieve documents that answer the query",
    "SciDocsRR-query": "Retrieve relevant paper titles",
    "SciDocsRR-document": "Retrieve relevant paper titles",
    "StackOverflowDupQuestions-query": "Retrieve semantically similar questions",
    "StackOverflowDupQuestions-document": "Retrieve semantically similar questions",
    "CmedqaRetrieval": "Given a query, retrieve documents that answer the query",
    "CovidRetrieval": "Given a query, retrieve documents that answer the query",
    "DuRetrieval": "Given a query, retrieve documents that answer the query",
    "EcomRetrieval": "Given a query, retrieve documents that answer the query",
    "MedicalRetrieval": "Given a query, retrieve documents that answer the query",
    "MMarcoRetrieval": "Given a query, retrieve documents that answer the query",
    "T2Retrieval": "Given a query, retrieve documents that answer the query",
    "VideoRetrieval": "Given a query, retrieve documents that answer the query",
    "MSMARCO": "Given a query, retrieve documents that answer the query",
    "ArguAna": "Given a query, retrieve documents that answer the query",
    "ClimateFEVER": "Given a query, retrieve documents that answer the query",
    "CQADupstackAndroidRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackAndroidRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackEnglishRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackEnglishRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGamingRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGamingRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGisRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGisRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackMathematicaRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackMathematicaRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackPhysicsRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackPhysicsRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackProgrammersRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackProgrammersRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackStatsRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackStatsRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackTexRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackTexRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackUnixRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackUnixRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackWebmastersRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackWebmastersRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackWordpressRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackWordpressRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "DBPedia": "Given a query, retrieve documents that answer the query",
    "FEVER": "Given a query, retrieve documents that answer the query",
    "FiQA2018": "Given a query, retrieve documents that answer the query",
    "HotpotQA": "Given a query, retrieve documents that answer the query",
    "NFCorpus": "Given a query, retrieve documents that answer the query",
    "NQ": "Given a query, retrieve documents that answer the query",
    "QuoraRetrieval-query": "Retrieve semantically similar questions",
    "QuoraRetrieval-document": "Retrieve semantically similar questions",
    "SCIDOCS-query": "Given a query, retrieve documents that answer the query",
    "SCIDOCS-document": "Given a query, retrieve documents that answer the query",
    "SciFact": "Given a query, retrieve documents that answer the query",
    "Touche2020": "Given a query, retrieve documents that answer the query",
    "TRECCOVID": "Given a query, retrieve documents that answer the query",
    "AFQMC-query": "Retrieve semantically similar text",
    "AFQMC-document": "Retrieve semantically similar text",
    "ATEC-query": "Retrieve semantically similar text",
    "ATEC-document": "Retrieve semantically similar text",
    "BQ-query": "Retrieve semantically similar text",
    "BQ-document": "Retrieve semantically similar text",
    "LCQMC-query": "Retrieve semantically similar text",
    "LCQMC-document": "Retrieve semantically similar text",
    "PAWSX-query": "Retrieve semantically similar text",
    "PAWSX-document": "Retrieve semantically similar text",
    "QBQTC-query": "Retrieve semantically similar text",
    "QBQTC-document": "Retrieve semantically similar text",
    "STSB-query": "Retrieve semantically similar text",
    "STSB-document": "Retrieve semantically similar text",
    "BIOSSES-query": "Retrieve semantically similar text",
    "BIOSSES-document": "Retrieve semantically similar text",
    "SICK-R-query": "Retrieve semantically similar text",
    "SICK-R-document": "Retrieve semantically similar text",
    "STS12-query": "Retrieve semantically similar text",
    "STS12-document": "Retrieve semantically similar text",
    "STS13-query": "Retrieve semantically similar text",
    "STS13-document": "Retrieve semantically similar text",
    "STS14-query": "Retrieve semantically similar text",
    "STS14-document": "Retrieve semantically similar text",
    "STS15-query": "Retrieve semantically similar text",
    "STS15-document": "Retrieve semantically similar text",
    "STS16-query": "Retrieve semantically similar text",
    "STS16-document": "Retrieve semantically similar text",
    "STS17-query": "Retrieve semantically similar text",
    "STS17-document": "Retrieve semantically similar text",
    "STS22-query": "Retrieve semantically similar text",
    "STS22-document": "Retrieve semantically similar text",
    "STSBenchmark-query": "Retrieve semantically similar text",
    "STSBenchmark-document": "Retrieve semantically similar text",
    "SummEval-query": "Retrieve semantically similar summaries",
    "SummEval-document": "Retrieve semantically similar summaries",
}

KaLM_Embedding_gemma_3_12b_task_prompts = {
    "AmazonCounterfactualClassification": "Classify a given Amazon customer review text as either counterfactual or not-counterfactual",
    "AmazonPolarityClassification": "Classify Amazon reviews into positive or negative sentiment",
    "AmazonReviewsClassification": "Classify the given Amazon review into its appropriate rating category",
    "Banking77Classification": "Given a online banking query, find the corresponding intents",
    "EmotionClassification": "Classify the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise",
    "ImdbClassification": "Classify the sentiment expressed in the given movie review text from the IMDB dataset",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios",
    "MTOPDomainClassification": "Classify the intent domain of the given utterance in task-oriented conversation",
    "MTOPIntentClassification": "Classify the intent of the given utterance in task-oriented conversation",
    "ToxicConversationsClassification": "Classify the given comments as either toxic or not toxic",
    "TweetSentimentExtractionClassification": "Classify the sentiment of a given tweet as either positive, negative, or neutral",
    "TNews": "Classify the fine-grained category of the given news title",
    "IFlyTek": "Given an App description text, find the appropriate fine-grained category",
    "MultilingualSentiment": "Classify sentiment of the customer review into positive, neutral, or negative",
    "JDReview": "Classify the customer review for iPhone on e-commerce platform into positive or negative",
    "OnlineShopping": "Classify the customer review for online shopping into positive or negative",
    "Waimai": "Classify the customer review from a food takeaway platform into positive or negative",
    "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArxivClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BiorxivClusteringP2P": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringP2P.v2": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringS2S": "Identify the main category of Biorxiv papers based on the titles",
    "BiorxivClusteringS2S.v2": "Identify the main category of Biorxiv papers based on the titles",
    "MedrxivClusteringP2P": "Identify the main category of Medrxiv papers based on the titles and abstracts",
    "MedrxivClusteringP2P.v2": "Identify the main category of Medrxiv papers based on the titles and abstracts",
    "MedrxivClusteringS2S": "Identify the main category of Medrxiv papers based on the titles",
    "MedrxivClusteringS2S.v2": "Identify the main category of Medrxiv papers based on the titles",
    "RedditClustering": "Identify the topic or theme of Reddit posts based on the titles",
    "RedditClusteringP2P": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "StackExchangeClustering": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClustering.v2": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClusteringP2P": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "StackExchangeClusteringP2P.v2": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "TwentyNewsgroupsClustering": "Identify the topic or theme of the given news articles",
    "CLSClusteringS2S": "Identify the main category of scholar papers based on the titles",
    "CLSClusteringP2P": "Identify the main category of scholar papers based on the titles and abstracts",
    "CLSClusteringP2P.v2": "Identify the main category of scholar papers based on the titles and abstracts",
    "ThuNewsClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "ThuNewsClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    "AskUbuntuDupQuestions-query": "Retrieve duplicate questions from AskUbuntu forum",
    "MindSmallReranking-query": "Retrieve relevant news articles based on user browsing history",
    "SciDocsRR-query": "Given a title of a scientific paper, retrieve the titles of other relevant papers",
    "StackOverflowDupQuestions-query": "Retrieve duplicate questions from StackOverflow forum",
    "SprintDuplicateQuestions": "Retrieve semantically duplicate questions",
    "TwitterSemEval2015": "Retrieve tweets that are semantically similar to the given tweet",
    "TwitterURLCorpus": "Retrieve tweets that are semantically similar to the given tweet",
    "T2Reranking-query": "Given a Chinese search query, retrieve web passages that answer the question",
    "MMarcoReranking-query": "Given a Chinese search query, retrieve web passages that answer the question",
    "CMedQAv1-reranking-query": "Given a Chinese community medical question, retrieve replies that best answer the question",
    "CMedQAv2-reranking-query": "Given a Chinese community medical question, retrieve replies that best answer the question",
    "Ocnli": "Retrieve semantically similar text.",
    "Cmnli": "Retrieve semantically similar text.",
    "ArguAna-query": "Given a claim, find documents that refute the claim",
    "ArguAna-document": "Given a claim, find documents that refute the claim",
    "ClimateFEVER-query": "Given a claim about climate change, retrieve documents that support or refute the claim",
    "ClimateFEVERHardNegatives-query": "Given a claim about climate change, retrieve documents that support or refute the claim",
    "DBPedia-query": "Given a query, retrieve relevant entity descriptions from DBPedia",
    "FEVER-query": "Given a claim, retrieve documents that support or refute the claim",
    "FEVERHardNegatives-query": "Given a claim, retrieve documents that support or refute the claim",
    "FiQA2018-query": "Given a financial question, retrieve user replies that best answer the question",
    "HotpotQA-query": "Given a multi-hop question, retrieve documents that can help answer the question",
    "HotpotQAHardNegatives-query": "Given a multi-hop question, retrieve documents that can help answer the question",
    "MSMARCO-query": "Given a web search query, retrieve relevant passages that answer the query",
    "NFCorpus-query": "Given a question, retrieve relevant documents that best answer the question",
    "NQ-query": "Given a question, retrieve Wikipedia passages that answer the question",
    "QuoraRetrieval-query": "Given a question, retrieve questions that are semantically equivalent to the given question",
    "SCIDOCS-query": "Given a title of a scientific paper, retrieve the titles of other relevant papers",
    "SciFact-query": "Given a scientific claim, retrieve documents that support or refute the claim",
    "Touche2020-query": "Given a question, retrieve detailed and persuasive arguments that answer the question",
    "Touche2020Retrieval.v3-query": "Given a question, retrieve detailed and persuasive arguments that answer the question",
    "TRECCOVID-query": "Given a medical query, retrieve documents that answer the query",
    "T2Retrieval-query": "Given a Chinese search query, retrieve web passages that answer the question",
    "MMarcoRetrieval-query": "Given a web search query, retrieve relevant passages that answer the query",
    "VoyageMMarcoReranking-query": "Given a Japanese search query, retrieve web passages that answer the question",
    "DuRetrieval-query": "Given a Chinese search query, retrieve web passages that answer the question",
    "CovidRetrieval-query": "Given a question on COVID-19, retrieve news articles that answer the question",
    "CmedqaRetrieval-query": "Given a Chinese community medical question, retrieve replies that best answer the question",
    "EcomRetrieval-query": "Given a user query from an e-commerce website, retrieve description sentences of relevant products",
    "MedicalRetrieval-query": "Given a medical question, retrieve user replies that best answer the question",
    "VideoRetrieval-query": "Given a video search query, retrieve the titles of relevant videos",
    "STSBenchmarkMultilingualSTS": "Retrieve semantically similar text",
    "SICKFr": "Retrieve semantically similar text",
    "SummEvalFr": "Given a news summary, retrieve other semantically similar summaries",
    "MasakhaNEWSClassification": "Classify the News in the given texts into one of the seven category: politics,sports,health,business,entertainment,technology,religion ",
    "OpusparcusPC": "Retrieve semantically similar text",
    "PAWSX": "Retrieve semantically similar text",
    "HALClusteringS2S": "Identify the main category of academic passage based on the titles and contents",
    "MasakhaNEWSClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    "MasakhaNEWSClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "MLSUMClusteringP2P": "Identify the topic or theme of the given articles based on the titles and contents",
    "MLSUMClusteringS2S": "Identify the topic or theme of the given articles based on the titles",
    "SyntecReranking-query": "Given a question, retrieve passages that answer the question",
    "AlloprofReranking-query": "Given a question, retrieve passages that answer the question",
    "AlloprofRetrieval-query": "Given a question, retrieve passages that answer the question",
    "BSARDRetrieval-query": "Given a question, retrieve passages that answer the question",
    "SyntecRetrieval-query": "Given a question, retrieve passages that answer the question",
    "XPQARetrieval-query": "Given a question, retrieve passages that answer the question",
    "MintakaRetrieval-query": "Given a question, retrieve passages that answer the question",
    "CBD": "Classify the sentiment of polish tweet reviews",
    "PolEmo2.0-IN": "Classify the sentiment of medicine and hotels online reviews",
    "PolEmo2.0-OUT": "Classify the sentiment of products and school online reviews",
    "AllegroReviews": "Classify the sentiment of reviews from e-commerce marketplace Allegro",
    "PAC": 'Classify Polish contract clauses into one of the following two types: "Safe Contract Clauses" and "Unfair Contract Clauses".',
    "SICK-E-PL": "Retrieve semantically similar text",
    "SICK-R-PL": "Retrieve semantically similar text",
    "STS22": "Retrieve semantically similar text",
    "AFQMC": "Retrieve semantically similar text",
    "BQ": "Retrieve semantically similar text",
    "LCQMC": "Retrieve semantically similar text",
    "QBQTC": "Retrieve semantically similar text",
    "STS12": "Retrieve semantically similar text",
    "PpcPC": "Retrieve semantically similar text",
    "CDSC-E": "Retrieve semantically similar text",
    "BornholmBitextMining": "Retrieve parallel sentences",
    "NorwegianCourtsBitextMining": "Retrieve parallel sentences",
    "PSC": "Retrieve semantically similar text",
    "EightTagsClustering": "Identify of headlines from social media posts in Polish  into 8 categories: film, history, food, medicine, motorization, work, sport and technology",
    "ArguAna-PL-query": "Given a claim, find documents that refute the claim",
    "DBPedia-PL-query": "Given a query, retrieve relevant entity descriptions from DBPedia",
    "FiQA-PL-query": "Given a financial question, retrieve user replies that best answer the question",
    "HotpotQA-PL-query": "Given a multi-hop question, retrieve documents that can help answer the question",
    "MSMARCO-PL-query": "Given a web search query, retrieve relevant passages that answer the query",
    "NFCorpus-PL-query": "Given a question, retrieve relevant documents that best answer the question",
    "NQ-PL-query": "Given a question, retrieve Wikipedia passages that answer the question",
    "Quora-PL-query": "Given a question, retrieve questions that are semantically equivalent to the given question",
    "SCIDOCS-PL-query": "Given a title of a scientific paper, retrieve the titles of other relevant papers",
    "SciFact-PL-query": "Given a scientific claim, retrieve documents that support or refute the claim",
    "TRECCOVID-PL-query": "Given a medical query, retrieve documents that answer the query",
    "GeoreviewClassification": "Classify the organization rating based on the reviews",
    "HeadlineClassification": "Classify the topic or theme of the given news headline",
    "InappropriatenessClassification": "Classify the given message as either sensitive topic or not",
    "KinopoiskClassification": "Classify the sentiment expressed in the given movie review text",
    "RuReviewsClassification": "Classify product reviews into positive, negative or neutral sentiment",
    "RuSciBenchGRNTIClassification": "Classify the category of scientific papers based on the titles and abstracts",
    "RuSciBenchOECDClassification": "Classify the category of scientific papers based on the titles and abstracts",
    "GeoreviewClusteringP2P": "Identify the organization category based on the reviews",
    "RuSciBenchGRNTIClusteringP2P": "Identify the category of scientific papers based on the titles and abstracts",
    "RuSciBenchOECDClusteringP2P": "Identify the category of scientific papers based on the titles and abstracts",
    "TERRa": "Given a premise, retrieve a hypothesis that is entailed by the premise",
    "RuBQReranking-query": "Given a question, retrieve Wikipedia passages that answer the question",
    "RiaNewsRetrieval-query": "Given a headline, retrieval relevant articles",
    "RuBQRetrieval-query": "Given a question, retrieve Wikipedia passages that answer the question",
    "RUParaPhraserSTS": "Retrieve semantically similar text",
    "RuSTSBenchmarkSTS": "Retrieve semantically similar text",
    "AppsRetrieval-query": "Given a question about code problem, retrieval code that can solve user's problem",
    "COIRCodeSearchNetRetrieval-query": "Given a code snippet, retrieve the comment corresponding to that code.",
    "CodeEditSearchRetrieval-query": "Given a piece of code, retrieval code that in the ",
    "CodeFeedbackMT-query": "Given a question about coding, retrieval code or passage that can solve user's question",
    "CodeFeedbackST-query": "Given a question about coding, retrieval code or passage that can solve user's question",
    "CodeSearchNetCCRetrieval-query": "Given a code comment, retrieve the code snippet corresponding to that comment.",
    "CodeSearchNetRetrieval-query": "Given a code snippet, retrieve the comment corresponding to that code.",
    "CodeTransOceanContest-query": "Given a piece for code, retrieval semantically similar code",
    "CodeTransOceanDL-query": "Given a piece for code, retrieval semantically similar code",
    "CosQA-query": "Given a question about coding, retrieval code or passage that can solve user's question",
    "StackOverflowQA-query": "Given a question about coding, retrieval code or passage that can solve user's question",
    "SyntheticText2SQL-query": "Given a user's question, retrieve SQL queries that are appropriate responses to the question",
    "BibleNLPBitextMining": "Retrieve parallel sentences",
    "BUCC.v2": "Retrieve parallel sentences",
    "DiaBlaBitextMining": "Retrieve parallel sentences",
    "FloresBitextMining": "Retrieve parallel sentences",
    "IN22GenBitextMining": "Retrieve parallel sentences",
    "IndicGenBenchFloresBitextMining": "Retrieve parallel sentences",
    "NollySentiBitextMining": "Retrieve parallel sentences",
    "NTREXBitextMining": "Retrieve parallel sentences",
    "NusaTranslationBitextMining": "Retrieve parallel sentences",
    "NusaXBitextMining": "Retrieve parallel sentences",
    "Tatoeba": "Retrieve parallel sentences",
    "BulgarianStoreReviewSentimentClassfication": "Classify user reviews into positive, negative or mixed sentiment",
    "CzechProductReviewSentimentClassification": "Classify product reviews into positive, neutral, or negative sentiment",
    "GreekLegalCodeClassification": "Given a greek legal text, classify its topic",
    "DBpediaClassification": "Given a Wikipedia articles, categorized it into classes based on its DBpedia ontology",
    "FinancialPhrasebankClassification": "Given financial news, categorized by sentiment into positive, negative, or neutral",
    "PoemSentimentClassification": "Gvien a poem, categorized by sentiment into positive, no_impact, negative or mixed",
    "TweetTopicSingleClassification": "Gvien a twitter, classify its topic",
    "EstonianValenceClassification": "Given a news article, categorized by sentiment into negatiivne, positiivne, neutraalne or vastuolulin",
    "FilipinoShopeeReviewsClassification": "Given a shop review, classify its rating on a scale from 1 to 5",
    "GujaratiNewsClassification": "Given a Gujarati news articles, classify ist topic",
    "SentimentAnalysisHindi": "Given a hindi text, categorized by sentiment into positive, negative or neutral",
    "IndonesianIdClickbaitClassification": "Given an Indonesian news headlines, classify its into clickbait or non-clickbait",
    "ItaCaseholdClassification": "Given a judgments, classify its topic",
    "KorSarcasmClassification": "Given a twitter, categorized it into sarcasm or not_sarcasm",
    "KurdishSentimentClassification": "Given a text, categorized by sentiment into positive or negative",
    "MacedonianTweetSentimentClassification": "Given a Macedonian tweet, categorized by sentiment into positive, negative, or neutral",
    "AfriSentiClassification": "Given a text, categorized by sentiment into positive, negative, or neutral",
    "CataloniaTweetClassification": "Given a tweet, categorized by sentiment into AGAINST, FAVOR or NEUTRAL",
    "CyrillicTurkicLangClassification": "Given a text, classify its language",
    "IndicLangClassification": "Given a text, classify its language",
    "MultiHateClassification": "Given a text, categorized by sentiment into hate or non-hate",
    "NusaParagraphEmotionClassification": "Given a paragraph, classify its emotion",
    "NusaX-senti": "Given a text, categorized by sentiment into positive or negative",
    "SwissJudgementClassification": "Given a news article, categorized it into approval or dismissal",
    "NepaliNewsClassification": "Given a news article, categorized it into business, entertainment or sports",
    "OdiaNewsClassification": "Given a news article, categorized it into business, entertainment or sports",
    "PunjabiNewsClassification": "Given a news article, categorized it into two-classes",
    "SinhalaNewsClassification": "Given a news article, categorized it into political, business, technology, sports and Entertainment",
    "CSFDSKMovieReviewSentimentClassification": "Given a movie review, classify its rating on a scale from 0 to 5",
    "SiswatiNewsClassification": "Given a news article in Siswati, classify its topic",
    "SlovakMovieReviewSentimentClassification": "Given a movie review, categorized it into positive or negative",
    "SwahiliNewsClassification": "Given a news article, classify its domain",
    "TswanaNewsClassification": "Given a news article, classify its topic",
    "IsiZuluNewsClassification": "Given a news article, classify its topic",
    "WikiCitiesClustering": "Identify of Wikipedia articles of cities by country",
    "RomaniBibleClustering": "Identify verses from the Bible in Kalderash Romani by book.",
    "ArXivHierarchicalClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArXivHierarchicalClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BigPatentClustering.v2": "Identify the category of documents from the Big Patent dataset",
    "AlloProfClusteringS2S": "Identify the topic of document titles from Allo Prof dataset",
    "AlloProfClusteringS2S.v2": "Identify the topic of document titles from Allo Prof dataset",
    "AlloProfClusteringP2P": "Identify the topic of document titles and descriptions from Allo Prof dataset",
    "HALClusteringS2S.v2": "Identify the topic of titles from HAL",
    "SIB200ClusteringS2S": "Identify the category of documents",
    "WikiClusteringP2P.v2": "Identify the category of wiki passages",
    "PlscClusteringP2P.v2": "Identify the category of titles+abstracts from Library of Science",
    "KorHateSpeechMLClassification": "Given a Korean online news comments, classify its fine-grained hate speech classes",
    "MalteseNewsClassification": "Given a maltese new, classify its topic",
    "MultiEURLEXMultilabelClassification": "Given a text, classify its topic",
    "BrazilianToxicTweetsClassification": "Classify the toxic tweets in Brazilian Portuguese into one of the six categories: LGBTQ+phobia, Xenophobia, Obscene, Insult, Misogyny and Racism.",
    "CTKFactsNLI": "Retrieve semantically similar text",
    "indonli": "Retrieve semantically similar text",
    "ArmenianParaphrasePC": "Retrieve semantically similar text",
    "PawsXPairClassification": "Retrieve semantically similar text",
    "RTE3": "Retrieve semantically similar text",
    "XNLI": "Retrieve semantically similar text",
    "GermanSTSBenchmark": "Retrieve semantically similar text",
    "SICK-R": "Retrieve semantically similar text",
    "STS13": "Retrieve semantically similar text",
    "STS14": "Retrieve semantically similar text",
    "STSBenchmark": "Retrieve semantically similar text",
    "FaroeseSTS": "Retrieve semantically similar text",
    "FinParaSTS": "Retrieve semantically similar text",
    "JSICK": "Retrieve semantically similar text",
    "IndicCrosslingualSTS": "Retrieve parallel sentences",
    "SemRel24STS": "Retrieve semantically similar text",
    "STS17": "Retrieve semantically similar text",
    "STS22.v2": "Retrieve semantically similar text",
    "STSES": "Retrieve semantically similar text",
    "STSB": "Retrieve semantically similar text",
    "AILAStatutes-query": "Identifying the most relevant statutes for a given situation",
    "HagridRetrieval-query": "Given an information-seeking question, retrieve the best replies to answer the question",
    "LegalBenchCorporateLobbying-query": "Given a query, retrieve relevant legal bill summaries",
    "LEMBPasskeyRetrieval-query": "Retrieval the relevant passage for the given query",
    "BelebeleRetrieval-query": "Retrieval the relevant passage for the given query",
    "MLQARetrieval-query": "Retrieval the relevant passage for the given query",
    "StatcanDialogueDatasetRetrieval-query": "Retrieval the relevant passage for the given query",
    "WikipediaRetrievalMultilingual-query": "Retrieval the relevant passage for the given query",
    "Core17InstructionRetrieval-query": "Retrieval the relevant passage for the given query with conditions",
    "News21InstructionRetrieval-query": "Retrieval the relevant passage for the given query with conditions",
    "Robust04InstructionRetrieval-query": "Retrieval the relevant passage for the given query with conditions",
    "WebLINXCandidatesReranking-query": "Retrieval the relevant passage for the given query",
    "WikipediaRerankingMultilingual-query": "Retrieval the relevant passage for the given query",
    "STS15": "Retrieve semantically similar text",
    "MIRACLRetrievalHardNegatives-query": "Retrieval relevant passage for the given query",
    "BIOSSES": "Retrieve semantically similar text",
    "CQADupstackRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGamingRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackGamingRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackUnixRetrieval-query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "CQADupstackUnixRetrieval-document": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
    "STS16": "Retrieve semantically similar text",
    "SummEval": "Retrieve semantically similar text",
    "ATEC": "Retrieve semantically similar text",
    "ScalaClassification": "Classify passages into correct or correct in Scandinavian Languages based on linguistic acceptability",
    "SpartQA-query": "Given the following spatial reasoning question, retrieve the right answer.",
    "CEDRClassification": "Given a comment as query, classify expressed emotions into joy, sadness, surprise, fear, and anger",
    "DalajClassification": "Classify texts based on linguistic acceptability in Swedish",
    "TempReasonL1-query": "Given the following question about time, retrieve the correct answer.",
    "WinoGrande-query": "Given the following sentence, retrieve an appropriate answer to fill in the missing underscored part.",
    "NordicLangClassification": "Classify texts based on language",
    "TwitterHjerneRetrieval-query": "Retrieve answers to questions asked in Danish tweets",
    "SwednClusteringP2P": "Identify news categories in Swedish passages",
}

KaLM_INSTRUCTION = "Instruct: {instruction} \n Query: "

HIT_TMG__KaLM_embedding_multilingual_mini_instruct_v1 = ModelMeta(
    loader=KALMWrapper,
    loader_kwargs=dict(
        instruction_template=KaLM_INSTRUCTION,
        max_seq_length=512,
        apply_instruction_to_passages=False,
        prompts_dict=KaLM_task_prompts,
    ),
    name="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1",
    model_type=["dense"],
    revision="45e42c89990c40aca042659133fc8b13c28634b5",
    release_date="2024-10-23",
    languages=["eng-Latn", "zho-Hans"],
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=kalm_training_data,  # Replace with actual dataset if available
    adapted_from="Qwen/Qwen2-0.5B",
    superseded_by=None,
    citation=KALM_EMBEDDING_CITATION,
)

HIT_TMG__KaLM_embedding_multilingual_mini_v1 = ModelMeta(
    loader=sentence_transformers_loader,
    name="HIT-TMG/KaLM-embedding-multilingual-mini-v1",
    model_type=["dense"],
    revision="8a82a0cd2b322b91723e252486f7cce6fd8ac9d3",
    release_date="2024-08-27",
    languages=["eng-Latn", "zho-Hans"],
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=kalm_training_data,
    adapted_from="Qwen/Qwen2-0.5B",
    superseded_by=None,
    citation=KALM_EMBEDDING_CITATION,
)

HIT_TMG__KaLM_embedding_multilingual_mini_instruct_v1_5 = ModelMeta(
    loader=KALMWrapper,
    loader_kwargs=dict(
        instruction_template=KaLM_INSTRUCTION,
        max_seq_length=512,
        apply_instruction_to_passages=False,
        prompts_dict=KaLM_task_prompts,
    ),
    name="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1.5",
    model_type=["dense"],
    revision="fcff2f8a54e4cd96b7766fef1ee960a43d42bb3c",
    release_date="2024-12-26",
    languages=["eng-Latn", "zho-Hans"],
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1.5",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=kalm_training_data,
    adapted_from="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1",
    superseded_by=None,
    citation=KALM_EMBEDDING_CITATION,
)

HIT_TMG__KaLM_embedding_multilingual_mini_instruct_v2 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=KaLM_INSTRUCTION,
        max_seq_length=512,
        apply_instruction_to_passages=False,
        prompts_dict=KaLM_v2_task_prompts,
    ),
    name="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v2",
    model_type=["dense"],
    revision="d2a21c232dc712ae8230af56d1027cf21b7864bf",
    release_date="2025-06-25",
    languages=["eng-Latn", "zho-Hans"],
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=942,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v2",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=kalm_v2_training_data,
    adapted_from="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1.5",
    superseded_by=None,
    citation=KALM_EMBEDDING_CITATION,
)

KaLM_Embedding_KaLM_embedding_multilingual_mini_instruct_v2_5 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=KaLM_INSTRUCTION,
        max_seq_length=512,
        apply_instruction_to_passages=False,
        prompts_dict=KaLM_v2_task_prompts,
    ),
    name="KaLM-Embedding/KaLM-embedding-multilingual-mini-instruct-v2.5",
    model_type=["dense"],
    revision="6a4cfc1084cb459ebd4729b53a8656a61448c720",
    release_date="2025-09-30",
    languages=["eng-Latn", "zho-Hans"],
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/KaLM-Embedding/KaLM-embedding-finetuning-data",
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/KaLM-Embedding/KaLM-embedding-multilingual-mini-instruct-v2.5",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=kalm_v2_training_data,
    adapted_from="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v2",
    superseded_by=None,
    citation="""@misc{zhao2025kalmembeddingv2,
      title={KaLM-Embedding-V2: Superior Training Techniques and Data Inspire A Versatile Embedding Model},
      author={Xinping Zhao and Xinshuo Hu and Zifei Shan and Shouzheng Huang and Yao Zhou and Xin Zhang and Zetian Sun and Zhenyu Liu and Dongfang Li and Xinyuan Wei and Youcheng Pan and Yang Xiang and Meishan Zhang and Haofen Wang and Jun Yu and Baotian Hu and Min Zhang},
      year={2025},
      eprint={2506.20923},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2506.20923},
}

@misc{hu2025kalmembedding,
      title={KaLM-Embedding: Superior Training Data Brings A Stronger Embedding Model},
      author={Xinshuo Hu and Zifei Shan and Xinping Zhao and Zetian Sun and Zhenyu Liu and Dongfang Li and Shaolin Ye and Xinyuan Wei and Qian Chen and Baotian Hu and Haofen Wang and Jun Yu and Min Zhang},
      year={2025},
      eprint={2501.01028},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2501.01028},
}""",
)

KaLM_Embedding_gemma_3_12b_2511 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=KaLM_INSTRUCTION,
        max_seq_length=512,
        apply_instruction_to_passages=True,
        prompts_dict=KaLM_Embedding_gemma_3_12b_task_prompts,
    ),
    name="tencent/KaLM-Embedding-Gemma3-12B-2511",
    model_type=["dense"],
    revision="edf22f4753f58b05e3f5495818d31f12db63056d",
    languages=None,
    open_weights=True,
    release_date="2025-11-06",
    n_parameters=11.76 * 1e9,
    n_embedding_parameters=None,
    memory_usage_mb=44884,
    max_tokens=32768,
    embed_dim=3840,
    license=None,
    reference="https://kalm-embedding.github.io/",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=True,
    public_training_code="https://github.com/HITsz-TMG/KaLM-Embedding",
    public_training_data=None,
    training_datasets=KaLM_Embedding_gemma_3_12b_training_data,
    citation="""@misc{zhao2025kalmembeddingv2,
      title={KaLM-Embedding-V2: Superior Training Techniques and Data Inspire A Versatile Embedding Model},
      author={Xinping Zhao and Xinshuo Hu and Zifei Shan and Shouzheng Huang and Yao Zhou and Xin Zhang and Zetian Sun and Zhenyu Liu and Dongfang Li and Xinyuan Wei and Youcheng Pan and Yang Xiang and Meishan Zhang and Haofen Wang and Jun Yu and Baotian Hu and Min Zhang},
      year={2025},
      eprint={2506.20923},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2506.20923},
}

@misc{hu2025kalmembedding,
      title={KaLM-Embedding: Superior Training Data Brings A Stronger Embedding Model},
      author={Xinshuo Hu and Zifei Shan and Xinping Zhao and Zetian Sun and Zhenyu Liu and Dongfang Li and Shaolin Ye and Xinyuan Wei and Qian Chen and Baotian Hu and Haofen Wang and Jun Yu and Min Zhang},
      year={2025},
      eprint={2501.01028},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2501.01028},
}""",
)



from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_LANGS = {
    "french": ["fra-Latn"],
    "spanish": ["spa-Latn"],
    "english": ["eng-Latn"],
    "german": ["deu-Latn"],
}


def _load_data(
    path: str,
    splits: list[str],
    langs: list | None = None,
    revision: str | None = None,
    num_proc: int = 1,
):
    if langs is None:
        corpus = {}
        queries = {}
        relevant_docs = {}
    else:
        corpus = {lang: {} for lang in langs}
        queries = {lang: {} for lang in langs}
        relevant_docs = {lang: {} for lang in langs}

    for split in splits:
        query_ds = load_dataset(
            path,
            "queries",
            split=split,
            revision=revision,
            num_proc=num_proc,
        )
        query_ds = query_ds.map(
            lambda x: {
                "id": f"query-{split}-{x['query-id']}",
                "text": x["query"],
                "modality": "text",
            },
            remove_columns=["query-id", "query"],
            num_proc=num_proc,
        )

        corpus_ds = load_dataset(
            path,
            "corpus",
            split=split,
            revision=revision,
            num_proc=num_proc,
        )
        corpus_ds = corpus_ds.map(
            lambda x: {
                "id": f"corpus-{split}-{x['corpus-id']}",
                "modality": "image",
            },
            remove_columns=["corpus-id"],
            num_proc=num_proc,
        )
        corpus_ds = corpus_ds.select_columns(["id", "image"])

        qrels_ds = load_dataset(
            path,
            "qrels",
            split=split,
            revision=revision,
            num_proc=num_proc,
        )

        if langs is None:
            queries[split] = query_ds.select_columns(["id", "text"])
            corpus[split] = corpus_ds
            relevant_docs[split] = {}
            for row in qrels_ds:
                qid = f"query-{split}-{row['query-id']}"
                did = f"corpus-{split}-{row['corpus-id']}"
                if qid not in relevant_docs[split]:
                    relevant_docs[split][qid] = {}
                relevant_docs[split][qid][did] = int(row["score"])
        else:
            for lang in langs:
                filtered_query_ds = query_ds.filter(lambda x: x["language"] == lang)
                queries[lang][split] = filtered_query_ds.select_columns(["id", "text"])

                corpus[lang][split] = corpus_ds

                relevant_docs[lang][split] = {}
                for row in qrels_ds:
                    qid = f"query-{split}-{row['query-id']}"
                    did = f"corpus-{split}-{row['corpus-id']}"
                    if qid not in relevant_docs[lang][split]:
                        relevant_docs[lang][split][qid] = {}
                    relevant_docs[lang][split][qid][did] = int(row["score"])

    return corpus, queries, relevant_docs


class Vidore2ESGReportsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Vidore2ESGReportsRetrieval",
        description="Retrieve associated pages according to questions.",
        reference="https://arxiv.org/pdf/2407.01449",
        dataset={
            "path": "vidore/esg_reports_v2",
            "revision": "0542c0d03da0ec1c8cbc517c8d78e7e95c75d3d3",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="ndcg_at_5",
        date=("2025-01-01", "2025-03-01"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{mace2025vidorev2,
  author = {Macé, Quentin and Loison António and Faysse, Manuel},
  journal = {arXiv preprint arXiv:2505.17166},
  title = {ViDoRe Benchmark V2: Raising the Bar for Visual Retrieval},
  year = {2025},
}
""",
        prompt={"query": "Find a screenshot that relevant to the user's question."},
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            langs=_LANGS.keys(),
            revision=self.metadata.dataset["revision"],
            num_proc=num_proc,
        )

        self.data_loaded = True


class Vidore2EconomicsReportsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Vidore2EconomicsReportsRetrieval",
        description="Retrieve associated pages according to questions.",
        reference="https://arxiv.org/pdf/2407.01449",
        dataset={
            "path": "vidore/economics_reports_v2",
            "revision": "b3e3a04b07fbbaffe79be49dabf92f691fbca252",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="ndcg_at_5",
        date=("2025-01-01", "2025-03-01"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{mace2025vidorev2,
  author = {Macé, Quentin and Loison António and Faysse, Manuel},
  journal = {arXiv preprint arXiv:2505.17166},
  title = {ViDoRe Benchmark V2: Raising the Bar for Visual Retrieval},
  year = {2025},
}
""",
        prompt={"query": "Find a screenshot that relevant to the user's question."},
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            langs=_LANGS.keys(),
            revision=self.metadata.dataset["revision"],
            num_proc=num_proc,
        )

        self.data_loaded = True


class Vidore2BioMedicalLecturesRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Vidore2BioMedicalLecturesRetrieval",
        description="Retrieve associated pages according to questions.",
        reference="https://arxiv.org/pdf/2407.01449",
        dataset={
            "path": "vidore/biomedical_lectures_v2",
            "revision": "a29202f0da409034d651614d87cd8938d254e2ea",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="ndcg_at_5",
        date=("2025-01-01", "2025-03-01"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{mace2025vidorev2,
  author = {Macé, Quentin and Loison António and Faysse, Manuel},
  journal = {arXiv preprint arXiv:2505.17166},
  title = {ViDoRe Benchmark V2: Raising the Bar for Visual Retrieval},
  year = {2025},
}
""",
        prompt={"query": "Find a screenshot that relevant to the user's question."},
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            langs=_LANGS.keys(),
            revision=self.metadata.dataset["revision"],
            num_proc=num_proc,
        )

        self.data_loaded = True


class Vidore2ESGReportsHLRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Vidore2ESGReportsHLRetrieval",
        description="Retrieve associated pages according to questions.",
        reference="https://arxiv.org/pdf/2407.01449",
        dataset={
            "path": "vidore/esg_reports_human_labeled_v2",
            "revision": "6d467dedb09a75144ede1421747e47cf036857dd",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_5",
        date=("2025-01-01", "2025-03-01"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{mace2025vidorev2,
  author = {Macé, Quentin and Loison António and Faysse, Manuel},
  journal = {arXiv preprint arXiv:2505.17166},
  title = {ViDoRe Benchmark V2: Raising the Bar for Visual Retrieval},
  year = {2025},
}
""",
        prompt={"query": "Find a screenshot that relevant to the user's question."},
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            revision=self.metadata.dataset["revision"],
            num_proc=num_proc,
        )

        self.data_loaded = True



from __future__ import annotations

from collections.abc import Mapping
from enum import Enum
from typing import TYPE_CHECKING, TypedDict

import numpy as np
import torch
from datasets import Dataset
from numpy.typing import NDArray

if TYPE_CHECKING:
    from PIL import Image
    from typing_extensions import NotRequired


class EncodeKwargs(TypedDict):
    """Keyword arguments for encoding methods.

    Attributes:
        batch_size: The batch size to use for encoding.
        show_progress_bar: Whether to show a progress bar during encoding.
    """

    batch_size: NotRequired[int]
    show_progress_bar: NotRequired[bool]


# --- Output types ---
Array = NDArray[np.floating | np.integer | np.bool] | torch.Tensor
"""General array type, can be a numpy array (float, int, or bool) or a torch tensor."""


# --- Input types ---
class PromptType(str, Enum):
    """The type of prompt used in the input for retrieval models. Used to differentiate between queries and documents.

    Attributes:
        query: A prompt that is a query.
        document: A prompt that is a document.
    """

    query = "query"
    document = "document"


class ConversationTurn(TypedDict):
    """A conversation, consisting of a list of messages.

    Attributes:
        role: The role of the message sender.
        content: The content of the message.
    """

    role: str
    content: str


Conversation = list[ConversationTurn]
"""A conversation, consisting of a list of messages."""


class TextInput(TypedDict):
    """The input to the encoder for text.

    Attributes:
        text: The text to encode. Can be a list of texts or a list of lists of texts.
    """

    text: list[str]


class CorpusInput(TextInput):
    """The input to the encoder for retrieval corpus.

    Attributes:
        title: The title of the text to encode. Can be a list of titles or a
            list of lists of titles.
        body: The body of the text to encode. Can be a list of bodies or a
            list of lists of bodies.
    """

    title: list[str]
    body: list[str]


class QueryInput(TextInput):
    """The input to the encoder for queries.

    Attributes:
        query: The query to encode. Can be a list of queries or a list of lists of queries.
        conversation: Optional. A list of conversations, each conversation is a list of messages.
        instruction: Optional. A list of instructions to encode.
    """

    query: list[str]
    conversation: NotRequired[list[Conversation]]
    instruction: NotRequired[list[str]]


class ImageInput(TypedDict):
    """The input to the encoder for images.

    Attributes:
        image: The image to encode. Can be a list of images or a list of lists of images.
    """

    image: list[Image.Image]


class AudioInput(TypedDict):
    """The input to the encoder for audio.

    Attributes:
        audio: The audio to encode. Can be a list of audio files or a list of lists of audio files.
    """

    audio: list[list[bytes]]


class MultimodalInput(TextInput, CorpusInput, QueryInput, ImageInput, AudioInput):  # type: ignore[misc]
    """The input to the encoder for multimodal data."""

    pass


BatchedInput = (
    TextInput | CorpusInput | QueryInput | ImageInput | AudioInput | MultimodalInput
)
"""
Represents the input format accepted by the encoder for a batch of data.

The encoder can process several input types depending on the task or modality.
Each type is defined as a separate structured input with its own fields.

### Supported input types

1. **[`TextInput`][mteb.types._encoder_io.TextInput]**
   For pure text inputs.

   ```python
   {"text": ["This is a sample text.", "Another text."]}
   ```
2. **[`CorpusInput`][mteb.types._encoder_io.CorpusInput]**
   For corpus-style inputs with titles and bodies.

   ```python
   {"text": ["Title 1 Body 1", "Title 2 Body 2"], "title": ["Title 1", "Title 2"], "body": ["Body 1", "Body 2"]}
   ```
3. **[`QueryInput`][mteb.types._encoder_io.QueryInput]**
   For query–instruction pairs, typically used in retrieval or question answering tasks. Queries and instructions are combined with the model's instruction template.

   ```python
   {
       "text": ["Instruction: Your task is to find document for this query. Query: What is AI?", "Instruction: Your task is to find term for definition. Query: Define machine learning."],
       "query": ["What is AI?", "Define machine learning."],
       "instruction": ["Your task is find document for this query.", "Your task is to find term for definition."]
   }
   ```
4. **[`ImageInput`][mteb.types._encoder_io.ImageInput]**
   For visual inputs consisting of images.

   ```python
   {"image": [PIL.Image1, PIL.Image2]}
   ```
5. **[`MultimodalInput`][mteb.types._encoder_io.MultimodalInput]**
   For combined text–image (multimodal) inputs.

   ```python
   {"text": ["This is a sample text."], "image": [PIL.Image1]}
   ```
"""


TextBatchedInput = TextInput | CorpusInput | QueryInput
"""The input to the encoder for a batch of text data."""

QueryDatasetType = Dataset
"""Retrieval query dataset, containing queries. Should have columns `id`, `text`."""
CorpusDatasetType = Dataset
"""Retrieval corpus dataset, containing documents. Should have columns `id`, `title`, `body`."""
InstructionDatasetType = Dataset
"""Retrieval instruction dataset, containing instructions. Should have columns `query-id`, `instruction`."""
RelevantDocumentsType = Mapping[str, Mapping[str, int]]
"""Relevant documents for each query, mapping query IDs to a mapping of document IDs and their relevance
scores. Should have columns `query-id`, `corpus-id`, `score`."""
TopRankedDocumentsType = Mapping[str, list[str]]
"""Top-ranked documents for each query, mapping query IDs to a list of document IDs. Should
have columns `query-id`, `corpus-ids`."""

RetrievalOutputType = dict[str, dict[str, float]]
"""Retrieval output, containing the scores for each query-document pair."""



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SketchyI2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SketchyI2IRetrieval",
        description="Retrieve photos from sketches.",
        reference="https://arxiv.org/abs/2202.01747",
        dataset={
            "path": "JamieSJS/sketchy",
            "revision": "c8b8c1b7a2f0a92f1bfaaa1c9afc22aa42c61d5b",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cv_recall_at_1",
        date=("2021-12-06", "2021-12-14"),  # conference dates
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{ypsilantis2021met,
  author = {Ypsilantis, Nikolaos-Antonios and Garcia, Noa and Han, Guangxing and Ibrahimi, Sarah and Van Noord, Nanne and Tolias, Giorgos},
  booktitle = {Thirty-fifth Conference on Neural Information Processing Systems Datasets and Benchmarks Track (Round 2)},
  title = {The met dataset: Instance-level recognition for artworks},
  year = {2021},
}
""",
    )
    skip_first_result = False



"""
SauerkrautLM Visual Document Retrieval Models - MTEB Integration

This module provides MTEB wrappers for SauerkrautLM ColPali-style models:
- SLM-ColQwen3 (Qwen3-VL backbone)
- SLM-ColLFM2 (LFM2 backbone)
- SLM-ColMinistral3 (Ministral3 backbone)

Based on:
- MTEB ColPali implementation: mteb/models/model_implementations/colpali_models.py
"""

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import (
    requires_image_dependencies,
    requires_package,
)
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_implementations.colpali_models import (
    COLPALI_CITATION,
    COLPALI_TRAINING_DATA,
)
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

logger = logging.getLogger(__name__)


SUPPORTED_LANGUAGES = [
    "eng-Latn",  # English
    "deu-Latn",  # German
    "fra-Latn",  # French
    "spa-Latn",  # Spanish
    "ita-Latn",  # Italian
    "por-Latn",  # Portuguese
]


class SLMBaseWrapper(AbsEncoder):
    """
    Base wrapper for SauerkrautLM multi-vector embedding models.

    All our models use late interaction (MaxSim) for retrieval scoring.
    """

    model_class = None
    processor_class = None
    model_name_prefix = "SLM"

    def __init__(
        self,
        model_name: str,
        revision: str | None = None,
        device: str | None = None,
        use_flash_attn: bool = True,
        **kwargs,
    ):
        requires_image_dependencies()
        requires_package(
            self, "sauerkrautlm_colpali", model_name, "pip install sauerkrautlm-colpali"
        )

        self.device = device or ("cuda" if torch.cuda.is_available() else "cpu")
        self._load_model_and_processor(model_name, revision, use_flash_attn, **kwargs)
        self.mdl = self.mdl.to(self.device)
        self.mdl.eval()

    def _load_model_and_processor(self, model_name, revision, use_flash_attn, **kwargs):
        """Override in subclasses to load specific model/processor."""
        raise NotImplementedError

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError("No text or image features found in inputs")

    def encode_input(self, inputs):
        """Forward pass through the model."""
        return self.mdl(**inputs)

    def _move_to_device(self, inputs: dict) -> dict:
        """Move all tensor inputs to the model's device."""
        result = {}
        for k, v in inputs.items():
            if isinstance(v, torch.Tensor):
                result[k] = v.to(self.device)
            else:
                result[k] = v
        return result

    def get_image_embeddings(
        self,
        images: DataLoader,
        batch_size: int = 32,
        **kwargs,
    ) -> torch.Tensor:
        import torchvision.transforms.functional as F

        all_embeds = []

        with torch.no_grad():
            for batch in tqdm(images, desc="Encoding images"):
                from PIL import Image

                imgs = [
                    F.to_pil_image(b) if not isinstance(b, Image.Image) else b
                    for b in batch["image"]
                ]
                inputs = self.processor.process_images(imgs)
                inputs = self._move_to_device(inputs)
                outs = self.encode_input(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def get_text_embeddings(
        self,
        texts: DataLoader,
        batch_size: int = 32,
        **kwargs,
    ) -> torch.Tensor:
        all_embeds = []

        with torch.no_grad():
            for batch in tqdm(texts, desc="Encoding texts"):
                inputs = self.processor.process_queries(batch["text"])
                inputs = self._move_to_device(inputs)
                outs = self.encode_input(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def calculate_probs(
        self,
        text_embeddings: torch.Tensor,
        image_embeddings: torch.Tensor,
    ) -> torch.Tensor:
        scores = self.similarity(text_embeddings, image_embeddings).T
        return scores.softmax(dim=-1)

    def similarity(
        self,
        a: torch.Tensor | list,
        b: torch.Tensor | list,
    ) -> torch.Tensor:
        return self.processor.score(a, b, device=self.device)


class SLMColQwen3Wrapper(SLMBaseWrapper):
    """Wrapper for SLM-ColQwen3 models (Qwen3-VL backbone)."""

    def _load_model_and_processor(self, model_name, revision, use_flash_attn, **kwargs):
        from sauerkrautlm_colpali.models.qwen3.colqwen3 import (
            ColQwen3,
            ColQwen3Processor,
        )

        self.mdl = ColQwen3.from_pretrained(
            model_name,
            torch_dtype=torch.bfloat16,
            attn_implementation="flash_attention_2" if use_flash_attn else "eager",
            revision=revision,
            **kwargs,
        )

        self.processor = ColQwen3Processor.from_pretrained(
            model_name,
            revision=revision,
        )

        logger.info(f"SLM-ColQwen3 loaded: dim={self.mdl.dim}, device={self.device}")


class SLMColLFM2Wrapper(SLMBaseWrapper):
    """Wrapper for SLM-ColLFM2 models (LFM2 backbone)."""

    def _load_model_and_processor(self, model_name, revision, use_flash_attn, **kwargs):
        from sauerkrautlm_colpali.models.lfm2.collfm2 import ColLFM2, ColLFM2Processor

        self.mdl = ColLFM2.from_pretrained(
            model_name,
            torch_dtype=torch.bfloat16,
            revision=revision,
            **kwargs,
        )

        self.processor = ColLFM2Processor.from_pretrained(
            model_name,
            revision=revision,
        )

        logger.info(f"SLM-ColLFM2 loaded: dim={self.mdl.dim}, device={self.device}")


class SLMColMinistral3Wrapper(SLMBaseWrapper):
    """Wrapper for SLM-ColMinistral3 models (Ministral3 backbone)."""

    def _load_model_and_processor(self, model_name, revision, use_flash_attn, **kwargs):
        from sauerkrautlm_colpali.models.ministral3.colministral3 import (
            ColMinistral3,
            ColMinistral3Processor,
        )

        self.mdl = ColMinistral3.from_pretrained(
            model_name,
            torch_dtype=torch.bfloat16,
        )

        self.processor = ColMinistral3Processor.from_pretrained(model_name)

        logger.info(
            f"SLM-ColMinistral3 loaded: dim={self.mdl.dim}, device={self.device}"
        )


SAUERKRAUTLM_CITATION = """
@misc{sauerkrautlm-colpali-2025,
  title={SauerkrautLM-ColPali: Multi-Vector Vision Retrieval Models},
  author={David Golchinfar},
  organization={VAGO Solutions},
  year={2025},
  url={https://github.com/VAGOsolutions/sauerkrautlm-colpali}
}
"""


slm_colqwen3_1_7b_turbo = ModelMeta(
    loader=SLMColQwen3Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColQwen3-1.7b-Turbo-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="19c295a18e057d6d82754f627c09408117ffdb66",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=1_756_572_288,
    memory_usage_mb=3350,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColQwen3-1.7b-Turbo-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="Qwen/Qwen3-VL-2B-Instruct",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)

slm_colqwen3_2b = ModelMeta(
    loader=SLMColQwen3Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColQwen3-2b-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="48f699713c10af754684e12060a2af9266462cc9",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=2_127_794_304,
    memory_usage_mb=4058,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColQwen3-2b-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="Qwen/Qwen3-VL-2B-Instruct",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)

slm_colqwen3_4b = ModelMeta(
    loader=SLMColQwen3Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColQwen3-4b-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="b635fbb3ab145f07608ed10a85def33544de1723",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=4_438_143_616,
    memory_usage_mb=8465,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColQwen3-4b-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="Qwen/Qwen3-VL-4B-Instruct",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)

slm_colqwen3_8b = ModelMeta(
    loader=SLMColQwen3Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColQwen3-8b-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="36ac136e451a7b8d8229725d69d4ec23aa4f03c8",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=8_145_318_256,
    memory_usage_mb=15536,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColQwen3-8b-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="Qwen/Qwen3-VL-8B-Instruct",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)

slm_collfm2_450m = ModelMeta(
    loader=SLMColLFM2Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColLFM2-450M-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="a65223fd6633f331ccff4483e47575c3c620dc60",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=450_953_856,
    memory_usage_mb=860,
    max_tokens=32768,
    embed_dim=128,
    license="https://huggingface.co/LiquidAI/LFM2-VL-450M/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColLFM2-450M-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="LiquidAI/LFM2-VL-450M",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)

slm_colministral3_3b = ModelMeta(
    loader=SLMColMinistral3Wrapper,
    name="VAGOsolutions/SauerkrautLM-ColMinistral3-3b-v0.1",
    languages=SUPPORTED_LANGUAGES,
    revision="54aa3ffbbce20471fdcc4afc07d13989c65e71b8",
    release_date="2025-12-20",
    modalities=["image", "text"],
    model_type=["late-interaction"],
    n_parameters=4_252_136_448,
    memory_usage_mb=8110,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["ColPali"],
    reference="https://huggingface.co/VAGOsolutions/SauerkrautLM-ColMinistral3-3b-v0.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    adapted_from="mistralai/Ministral-3B-Instruct-2410",
    training_datasets=COLPALI_TRAINING_DATA,
    citation=SAUERKRAUTLM_CITATION + COLPALI_CITATION,
)



import argparse
import logging
import os
import warnings
from pathlib import Path
from typing import TYPE_CHECKING

import torch
from rich.logging import RichHandler

import mteb
from mteb.cache import ResultCache
from mteb.cli._display_tasks import _display_benchmarks, _display_tasks
from mteb.cli.generate_model_card import generate_model_card
from mteb.evaluate import OverwriteStrategy

if TYPE_CHECKING:
    from mteb.abstasks.abstask import AbsTask
    from mteb.types import EncodeKwargs

logger = logging.getLogger(__name__)


def run(args: argparse.Namespace) -> None:
    """Run a model on a set of tasks."""
    # set logging based on verbosity level
    if args.verbosity == 0:
        logging.getLogger("mteb").setLevel(logging.CRITICAL)
    elif args.verbosity == 1:
        logging.getLogger("mteb").setLevel(logging.WARNING)
    elif args.verbosity == 2:
        logging.getLogger("mteb").setLevel(logging.INFO)
    elif args.verbosity == 3:
        logging.getLogger("mteb").setLevel(logging.DEBUG)

    if args.benchmarks and (
        args.tasks
        or args.eval_splits
        or args.languages
        or args.task_types
        or args.categories
    ):
        logger.warning(
            "`benchmarks` is specified but so is one or more of `tasks`, `eval_splits`, `languages`, `task_types` and `categories`. These will be ignored."
        )

    logger.debug("Setting environment variable TOKENIZERS_PARALLELISM to false")
    os.environ["TOKENIZERS_PARALLELISM"] = "false"

    logger.info("Running with parameters: %s", args)

    if args.device is None:
        device = "cuda" if torch.cuda.is_available() else "cpu"
    else:
        device = args.device

    model = mteb.get_model(args.model, args.model_revision, device=device)

    if args.benchmarks:
        benchmarks = mteb.get_benchmarks(names=args.benchmarks)
        tasks = tuple(t for b in benchmarks for t in b.tasks)
    else:
        tasks = mteb.get_tasks(
            categories=args.categories,
            task_types=args.task_types,
            languages=args.languages,
            tasks=args.tasks,
            eval_splits=args.eval_splits,
        )

    encode_kwargs: EncodeKwargs = {}
    if args.batch_size is not None:
        encode_kwargs["batch_size"] = args.batch_size

    overwrite_strategy = args.overwrite_strategy
    if args.overwrite:
        warnings.warn(
            "`--overwrite` is deprecated, please use `--overwrite-strategy 'always'` instead.",
            DeprecationWarning,
        )
        overwrite_strategy = OverwriteStrategy.ALWAYS.value

    prediction_folder = args.prediction_folder
    if args.save_predictions:
        warnings.warn(
            "`--save_predictions` is deprecated, please use `--prediction-folder` instead.",
            DeprecationWarning,
        )
        prediction_folder = args.output_folder

    mteb.evaluate(
        model,
        tasks,
        cache=ResultCache(args.output_folder),
        co2_tracker=args.co2_tracker,
        overwrite_strategy=overwrite_strategy,
        encode_kwargs=encode_kwargs,
        prediction_folder=prediction_folder,
    )


def _available_benchmarks(args: argparse.Namespace) -> None:
    benchmarks = mteb.get_benchmarks(names=args.benchmarks)
    _display_benchmarks(benchmarks)


def _available_tasks(args: argparse.Namespace) -> None:
    tasks = mteb.get_tasks(
        categories=args.categories,
        task_types=args.task_types,
        languages=args.languages,
        tasks=args.tasks,
    )
    _display_tasks(tasks)


def _add_benchmark_selection_args(parser: argparse.ArgumentParser) -> None:
    """Adds arguments to the parser for filtering benchmarks by name."""
    parser.add_argument(
        "-b",
        "--benchmarks",
        nargs="+",
        type=str,
        default=None,
        help="List of benchmark to be evaluated.",
    )


def _add_task_selection_args(parser: argparse.ArgumentParser) -> None:
    """Adds arguments to the parser for filtering tasks by type, category, language, and task name."""
    parser.add_argument(
        "--task-types",
        nargs="+",
        type=str,
        default=None,
        help="List of task types (Clustering, Retrieval..) to be evaluated. If None, the filter is not applied",
    )
    parser.add_argument(
        "--categories",
        nargs="+",
        type=str,
        default=None,
        help="List of task categories (s2s, p2p..) to be evaluated. If None the filter is not applied",
    )
    parser.add_argument(
        "-t",
        "--tasks",
        nargs="+",
        type=str,
        default=None,
        help="List of tasks to be evaluated. If specified, the other arguments are ignored.",
    )
    parser.add_argument(
        "-l",
        "--languages",
        nargs="*",
        type=str,
        default=None,
        help="List of languages to be evaluated. if not set, all languages will be evaluated. Specified as ISO 639-3 codes (e.g. eng, deu, fra).",
    )


def _add_available_tasks_parser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "available-tasks", help="List the available tasks within MTEB"
    )
    _add_task_selection_args(parser)

    parser.set_defaults(func=_available_tasks)


def _add_available_benchmarks_parser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "available-benchmarks", help="List the available benchmarks within MTEB"
    )
    _add_benchmark_selection_args(parser)

    parser.set_defaults(func=_available_benchmarks)


def _add_run_parser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser("run", help="Run a model on a set of tasks")

    parser.add_argument(
        "-m",
        "--model",
        type=str,
        help="Model to use. Will prioritize model implementation in MTEB's model registry, but default to loading the model using sentence-transformers.",
    )

    _add_task_selection_args(parser)
    _add_benchmark_selection_args(parser)

    parser.add_argument(
        "--device", type=int, default=None, help="Device to use for computation."
    )
    parser.add_argument(
        "--output-folder",
        "--output_folder",  # for backward compatibility
        type=str,
        default="results",
        help="Output directory for results. Will default to `results` if not set.",
    )
    parser.add_argument(
        "-v", "--verbosity", type=int, default=2, help="Verbosity level"
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--co2-tracker",
        dest="co2_tracker",
        action="store_true",
        help="Enable CO₂ tracker. If no argument is passed, the CO₂ tracker will be run if installed.",
    )
    group.add_argument(
        "--no-co2-tracker",
        dest="co2_tracker",
        action="store_false",
        help="Disable CO₂ tracker. If no argument is passed, the CO₂ tracker will be run if installed.",
    )
    parser.set_defaults(co2_tracker=None)

    parser.add_argument(
        "--eval-splits",
        "--eval_splits",  # for backward compatibility
        nargs="+",
        type=str,
        default=None,
        help="Evaluation splits to use (train, dev, test..). If None, all splits will be used.",
    )
    parser.add_argument(
        "--model-revision",
        "--model_revision",  # for backward compatibility
        type=str,
        default=None,
        help="Revision of the model to be loaded. Revisions are automatically read if the model is loaded from huggingface.",
    )
    parser.add_argument(
        "--batch-size",
        "--batch_size",  # for backward compatibility
        type=int,
        default=None,
        help="Batch size of the encode. Will be passed to the MTEB as `mteb.evaluate(model, task, encode_kwargs={'batch_size': value})`.",
    )
    # for backward compatibility
    parser.add_argument(
        "--overwrite",
        action="store_true",
        default=False,
        help="Whether to overwrite existing results. Deprecated, use `--overwrite-strategy 'always'` instead.",
    )
    parser.add_argument(
        "--overwrite-strategy",
        type=str,
        default="only-missing",
        choices=[val.value for val in OverwriteStrategy],
        help=(
            "Strategy for when to overwrite. Can be 'always', 'never', 'only-missing'. 'only-missing' will only rerun the missing splits of a task."
            + " It will not rerun the splits if the dataset revision or mteb version has changed."
        ),
    )
    parser.add_argument(
        "--save_predictions",
        action="store_true",
        default=False,
        help="Saves the predictions file in output_folder. Deprecated, use `--prediction-folder` instead.",
    )
    parser.add_argument(
        "--prediction-folder",
        type=str,
        default=None,
        help="Folder to save the model predictions in. If None, predictions will not be saved.",
    )

    parser.set_defaults(func=run)


def _create_meta(args: argparse.Namespace) -> None:
    model_name = args.model_name
    tasks_names = args.tasks
    benchmarks = args.benchmarks
    results_folder = Path(args.results_folder) if args.results_folder else None
    output_path = Path(args.output_path)
    overwrite = args.overwrite
    from_existing = args.from_existing if args.from_existing else None
    if from_existing is not None and from_existing.endswith(".md"):
        from_existing = Path(from_existing)

    if output_path.exists() and overwrite:
        msg = "Output path already exists, overwriting."
        logger.warning(msg)
        warnings.warn(msg)
    elif output_path.exists():
        raise FileExistsError(
            "Output path already exists, use --overwrite to overwrite."
        )

    benchmarks = None
    tasks: list[AbsTask] = []
    if tasks_names is not None:
        tasks = list(mteb.get_tasks(tasks_names))
    if benchmarks is not None:
        benchmarks = mteb.get_benchmarks(benchmarks)

    generate_model_card(
        model_name,
        tasks,
        benchmarks,
        existing_model_card_id_or_path=from_existing,
        results_cache=ResultCache(results_folder),
        output_path=output_path,
    )


def _add_create_meta_parser(subparsers) -> None:
    parser = subparsers.add_parser("create-model-results", help="Create model results")

    parser.add_argument(
        "--model-name",
        type=str,
        help="Name of the model",
    )
    parser.add_argument(
        "--tasks",
        type=str,
        nargs="+",
        help="Name of the tasks to use. By default, all tasks results will be used.",
        required=False,
        default=None,
    )
    parser.add_argument(
        "--benchmarks",
        type=str,
        nargs="+",
        help="Name of the benchmarks to use",
        required=False,
        default=None,
    )
    parser.add_argument(
        "--results-folder",
        type=str,
        help="Folder containing the results of a model run",
        required=False,
        default=None,
    )
    parser.add_argument(
        "--output-path",
        type=str,
        default="model_card.md",
        help="Output path for the model metadata",
    )
    parser.add_argument(
        "--overwrite",
        action="store_true",
        default=False,
        help="Overwrite the output file if it already exists",
    )
    parser.add_argument(
        "--from-existing",
        type=str,
        required=False,
        help="Merge results with existing README.md. Can be path to file or Huggingface model_id",
        default=None,
    )

    parser.set_defaults(func=_create_meta)


def _add_leaderboard_parser(subparsers) -> None:
    parser = subparsers.add_parser("leaderboard", help="Launch the MTEB leaderboard")

    parser.add_argument(
        "--cache-path",
        type=str,
        help="Path to the cache folder containing model results",
        required=False,
        default=None,
    )
    parser.add_argument(
        "--host",
        type=str,
        default="0.0.0.0",
        help="Host to run the leaderboard server on",
    )
    parser.add_argument(
        "--port",
        type=int,
        default=7860,
        help="Port to run the leaderboard server on",
    )
    parser.add_argument(
        "--share",
        action="store_true",
        default=False,
        help="Create a public URL for the leaderboard",
    )

    parser.set_defaults(func=_leaderboard)


def _leaderboard(args: argparse.Namespace) -> None:
    """Launch the MTEB leaderboard with specified cache path."""
    # Import leaderboard module only when needed to avoid requiring leaderboard dependencies
    # for other CLI commands
    try:
        import gradio as gr

        from mteb.leaderboard import get_leaderboard_app
    except ImportError as e:
        raise ImportError(
            "Seems like some dependencies are not installed. "
            + "You can likely install these using: `pip install mteb[leaderboard]`. "
            + f"{e}"
        )

    cache_path = args.cache_path

    if cache_path:
        logger.info(f"Using cache path: {cache_path}")
        cache = ResultCache(cache_path)
    else:
        cache = ResultCache()
        logger.info(f"Using default cache path: {cache.cache_path}")

    app = get_leaderboard_app(cache)

    logger.info(f"Starting leaderboard on {args.host}:{args.port}")
    if args.share:
        logger.info("Creating public URL...")

    logging.getLogger("mteb.load_results.task_results").setLevel(
        logging.ERROR
    )  # Warnings related to task split
    logging.getLogger("mteb.model_meta").setLevel(
        logging.ERROR
    )  # Warning related to model metadata (fetch_from_hf=False)
    logging.getLogger("mteb.load_results.benchmark_results").setLevel(
        logging.ERROR
    )  # Warning related to model metadata (fetch_from_hf=False)
    warnings.filterwarnings("ignore", message="Couldn't get scores for .* due to .*")

    # Head content for Tailwind CSS
    head = """
    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">
    """

    app.launch(
        server_name=args.host,
        server_port=args.port,
        share=args.share,
        theme=gr.themes.Soft(
            font=[gr.themes.GoogleFont("Roboto Mono"), "Arial", "sans-serif"],
        ),
        head=head,
    )


def build_cli() -> argparse.ArgumentParser:
    """Builds the argument parser for the MTEB CLI.

    Returns:
        An argparse.ArgumentParser object configured with subcommands and options.
    """
    parser = argparse.ArgumentParser(
        description="MTEB Command Line Interface",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    subparsers = parser.add_subparsers(
        title="subcommands", description="valid subcommands", help="additional help"
    )

    _add_run_parser(subparsers)
    _add_available_tasks_parser(subparsers)
    _add_available_benchmarks_parser(subparsers)
    _add_create_meta_parser(subparsers)
    _add_leaderboard_parser(subparsers)

    return parser


def main() -> None:
    """Main entry point for the MTEB CLI."""
    logging.basicConfig(
        level=logging.WARNING,
        format="%(message)s",
        handlers=[RichHandler(rich_tracebacks=True)],
    )

    parser = build_cli()
    args = parser.parse_args()
    args.func(args)



from mteb.models import sentence_transformers_loader
from mteb.models.model_meta import ModelMeta, ScoringFunction

sbert_swedish = ModelMeta(
    loader=sentence_transformers_loader,  # type: ignore[arg-type]
    name="KBLab/sentence-bert-swedish-cased",
    model_type=["dense"],
    languages=["swe-Latn"],
    open_weights=True,
    revision="6b5e83cd29c03729cfdc33d13b1423399b0efb5c",
    release_date="2023-01-11",
    n_parameters=124690944,
    n_embedding_parameters=38_649_600,
    memory_usage_mb=476,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=384,
    reference="https://huggingface.co/KBLab/sentence-bert-swedish-cased",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "safetensors",
        "GGUF",
        "Transformers",
    ],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    adapted_from="sentence-transformers/all-mpnet-base-v2",
    citation="""@misc{rekathati2021introducing,
  author = {Rekathati, Faton},
  title = {The KBLab Blog: Introducing a Swedish Sentence Transformer},
  url = {https://kb-labb.github.io/posts/2021-08-23-a-swedish-sentence-transformer/},
  year = {2021}
}""",
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, TypedDict, cast

from scipy.stats import pearsonr, spearmanr

from mteb._evaluators import AnySTSEvaluator
from mteb.models import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_image_statistics,
    calculate_score_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

if TYPE_CHECKING:
    from pathlib import Path

    from datasets import Dataset

    from mteb._evaluators.any_sts_evaluator import STSEvaluatorScores
    from mteb.models import MTEBModels
    from mteb.types import EncodeKwargs, PromptType
    from mteb.types.statistics import (
        ImageStatistics,
        ScoreStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class AnySTSDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for STS

    Attributes:
        num_samples: number of samples in the dataset.
        number_of_characters: Total number of symbols in the dataset.
        unique_pairs: Number of unique pairs

        text1_statistics: Statistics for sentence1
        text2_statistics: Statistics for sentence2

        image1_statistics: Statistics for image1
        image2_statistics: Statistics for image2

        label_statistics: Statistics for labels
    """

    num_samples: int
    number_of_characters: int | None
    unique_pairs: int | None

    text1_statistics: TextStatistics | None
    text2_statistics: TextStatistics | None

    image1_statistics: ImageStatistics | None
    image2_statistics: ImageStatistics | None

    label_statistics: ScoreStatistics


class STSMetrics(TypedDict):
    """Metrics for STS tasks

    Attributes:
        pearson: Pearson correlation coefficient using the model's similarity function or cosine similarity if not available
        spearman: Spearman correlation coefficient using the model's similarity function or cosine similarity if not available
        cosine_pearson: Pearson correlation coefficient using cosine similarity
        cosine_spearman: Spearman correlation coefficient using cosine similarity
        manhattan_pearson: Pearson correlation coefficient using Manhattan distance
        manhattan_spearman: Spearman correlation coefficient using Manhattan distance
        euclidean_pearson: Pearson correlation coefficient using Euclidean distance
        euclidean_spearman: Spearman correlation coefficient using Euclidean distance
    """

    pearson: float
    spearman: float
    cosine_pearson: float
    cosine_spearman: float
    manhattan_pearson: float
    manhattan_spearman: float
    euclidean_pearson: float
    euclidean_spearman: float


class AbsTaskSTS(AbsTask):
    """Abstract class for STS experiments.

    Attributes:
        dataset: Dataset or dict of Datasets for different subsets (e.g., languages). Dataset must contain columns specified in column_names and a 'score' column.
            Columns in column_names should contain the text or image data to be compared.
        column_names: Tuple containing the names of the two columns to compare.
        min_score: Minimum possible score in the dataset.
        max_score: Maximum possible score in the dataset.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
        input1_prompt_type: Type of prompt of first input. Used for asymmetric tasks.
        input2_prompt_type: Type of prompt of second input. Used for asymmetric tasks.
    """

    abstask_prompt = "Retrieve semantically similar text."
    column_names: tuple[str, str] = ("sentence1", "sentence2")
    min_score: int = 0
    max_score: int = 5
    input1_prompt_type: PromptType | None = None
    input2_prompt_type: PromptType | None = None

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int = 1,
        **kwargs: Any,
    ) -> STSMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        normalized_scores = list(map(self._normalize, data_split["score"]))
        data_split = data_split.select_columns(list(self.column_names))

        evaluator = AnySTSEvaluator(
            data_split,
            self.column_names,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            input1_prompt_type=self.input1_prompt_type,
            input2_prompt_type=self.input2_prompt_type,
            **kwargs,
        )
        scores = evaluator(
            model,
            encode_kwargs=encode_kwargs,
            num_proc=num_proc,
        )

        if prediction_folder:
            self._save_task_predictions(
                scores,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        return self._calculate_scores(scores, normalized_scores)

    def _calculate_scores(
        self, scores: STSEvaluatorScores, normalized_scores: list[float]
    ) -> STSMetrics:
        def compute_corr(x: list[float], y: list[float]) -> tuple[float, float]:
            """Return (pearson, spearman) correlations between x and y."""
            return float(pearsonr(x, y)[0]), float(spearmanr(x, y)[0])

        cosine_pearson, cosine_spearman = compute_corr(
            normalized_scores, scores["cosine_scores"]
        )
        manhattan_pearson, manhattan_spearman = compute_corr(
            normalized_scores, scores["manhattan_distances"]
        )
        euclidean_pearson, euclidean_spearman = compute_corr(
            normalized_scores, scores["euclidean_distances"]
        )

        if scores["similarity_scores"] is not None:
            pearson, spearman = compute_corr(
                normalized_scores, scores["similarity_scores"]
            )
        else:
            # if model does not have a similarity function, assume cosine similarity
            pearson, spearman = cosine_pearson, cosine_spearman

        return STSMetrics(
            # using the models own similarity score
            pearson=pearson,
            spearman=spearman,
            # generic similarity scores
            cosine_pearson=cosine_pearson,
            cosine_spearman=cosine_spearman,
            manhattan_pearson=manhattan_pearson,
            manhattan_spearman=manhattan_spearman,
            euclidean_pearson=euclidean_pearson,
            euclidean_spearman=euclidean_spearman,
        )

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> AnySTSDescriptiveStatistics:
        first_column, second_column = self.column_names
        self.dataset = cast("dict[str, dict[str, Dataset]]", self.dataset)

        if hf_subset:
            sentence1 = self.dataset[hf_subset][split][first_column]
            sentence2 = self.dataset[hf_subset][split][second_column]
            score = self.dataset[hf_subset][split]["score"]
        elif compute_overall:
            sentence1 = []
            sentence2 = []
            score = []
            for hf_subset in self.metadata.eval_langs:
                sentence1.extend(self.dataset[hf_subset][split][first_column])
                sentence2.extend(self.dataset[hf_subset][split][second_column])
                score.extend(self.dataset[hf_subset][split]["score"])
        else:
            sentence1 = self.dataset[split][first_column]
            sentence2 = self.dataset[split][second_column]
            score = self.dataset[split]["score"]

        if "text" in self.metadata.modalities:
            text1_statistics = calculate_text_statistics(sentence1)
            text2_statistics = calculate_text_statistics(sentence2)

            unique_pairs = len(set(zip(sentence1, sentence2)))
        else:
            text1_statistics = None
            text2_statistics = None
            unique_pairs = None

        if "image" in self.metadata.modalities:
            image1_statistics = calculate_image_statistics(sentence1)
            image2_statistics = calculate_image_statistics(sentence2)
        else:
            image1_statistics = None
            image2_statistics = None

        labels_statistics = calculate_score_statistics(score)

        return AnySTSDescriptiveStatistics(
            num_samples=len(sentence1),
            number_of_characters=(
                text1_statistics["total_text_length"]
                + text2_statistics["total_text_length"]
                if text1_statistics
                else None
            ),
            unique_pairs=unique_pairs,
            text1_statistics=text1_statistics,
            text2_statistics=text2_statistics,
            image1_statistics=image1_statistics,
            image2_statistics=image2_statistics,
            label_statistics=labels_statistics,
        )

    def _push_dataset_to_hub(self, repo_name: str, num_proc: int = 1) -> None:
        self._upload_dataset_to_hub(
            repo_name,
            [self.column_names[0], self.column_names[1], "score"],
            num_proc=num_proc,
        )

    def _normalize(self, x: float) -> float:
        return (x - self.min_score) / (self.max_score - self.min_score)



from __future__ import annotations

import json
import logging
from collections import defaultdict
from pathlib import Path
from time import time
from typing import TYPE_CHECKING, Any, Literal

from datasets import Dataset, DatasetDict, concatenate_datasets

from mteb._create_dataloaders import (
    _combine_queries_with_instruction_text,
    _convert_conv_history_to_query,
    _corpus_to_dict,
)
from mteb._evaluators import RetrievalEvaluator
from mteb._evaluators.retrieval_metrics import make_score_dict
from mteb.models import (
    CrossEncoderProtocol,
    EncoderProtocol,
    SearchCrossEncoderWrapper,
    SearchEncoderWrapper,
    SearchProtocol,
)
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_image_statistics,
    calculate_relevant_docs_statistics,
    calculate_text_statistics,
    calculate_top_ranked_statistics,
)
from .abstask import AbsTask
from .retrieval_dataset_loaders import (
    RetrievalDatasetLoader,
    RetrievalSplitData,
    _combine_queries_with_instructions_datasets,
)

if TYPE_CHECKING:
    from collections.abc import Callable, Mapping, Sequence

    from typing_extensions import Self

    from mteb.models import (
        MTEBModels,
    )
    from mteb.types import (
        EncodeKwargs,
        HFSubset,
        QueryDatasetType,
        RelevantDocumentsType,
        RetrievalOutputType,
        ScoresDict,
    )
    from mteb.types.statistics import (
        ImageStatistics,
        RelevantDocsStatistics,
        TextStatistics,
        TopRankedStatistics,
    )


logger = logging.getLogger(__name__)


class RetrievalDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for Retrieval

    Attributes:
        num_samples: Number of queries and documents
        number_of_characters: Total number of characters in queries and documents

        documents_text_statistics: Statistics for documents
        documents_image_statistics: Statistics for documents
        queries_text_statistics: Statistics for queries
        queries_image_statistics: Statistics for queries
        relevant_docs_statistics: Statistics for relevant documents
        top_ranked_statistics: Statistics for top ranked documents (if available)
    """

    num_samples: int
    number_of_characters: int

    documents_text_statistics: TextStatistics | None
    documents_image_statistics: ImageStatistics | None
    queries_text_statistics: TextStatistics | None
    queries_image_statistics: ImageStatistics | None

    relevant_docs_statistics: RelevantDocsStatistics

    # this is for datasets that do reranking
    top_ranked_statistics: TopRankedStatistics | None


def _filter_queries_without_positives(
    relevant_docs: RelevantDocumentsType, queries: QueryDatasetType
) -> tuple[RelevantDocumentsType, QueryDatasetType]:
    _relevant_docs = {}
    for idx in relevant_docs:
        if len(relevant_docs[idx]) == 0:  # no relevant docs
            continue
        _relevant_docs[idx] = relevant_docs[idx]

    queries = queries.filter(
        lambda x: x["id"] in _relevant_docs.keys(), desc="Filtering queries by qrels"
    )

    return _relevant_docs, queries


class AbsTaskRetrieval(AbsTask):
    """Abstract class for retrieval experiments.

    Attributes:
        dataset: A nested dictionary where the first key is the subset (language or "default"),
                 the second key is the split (e.g., "train", "test"), and the value is a RetrievalSplitData object.
        ignore_identical_ids: If True, identical IDs in queries and corpus are ignored during evaluation.
        k_values: A sequence of integers representing the k values for evaluation metrics.
        skip_first_result: If True, the first result is skipped during evaluation
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    ignore_identical_ids: bool = False
    abstask_prompt = "Retrieve text based on user query."
    k_values: Sequence[int] = (1, 3, 5, 10, 20, 100, 1000)
    _top_k: int = max(k_values)
    dataset: dict[str, dict[str, RetrievalSplitData]]
    _support_cross_encoder: bool = True
    _support_search: bool = True
    _previous_results_model_meta: dict[str, Any] | None = None
    skip_first_result: bool = False

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        empty_dataset = Dataset.from_dict({})
        self.dataset = defaultdict(
            lambda: defaultdict(
                lambda: RetrievalSplitData(
                    corpus=empty_dataset,
                    queries=empty_dataset,
                    relevant_docs={},
                    top_ranked=None,
                )
            )
        )

    def convert_v1_dataset_format_to_v2(self, num_proc: int) -> None:
        """Convert dataset from v1 (from `self.queries`, `self.document`) format to v2 format (`self.dotaset`)."""
        # check if dataset is `v1` version
        if not hasattr(self, "queries"):
            return
        empty_dataset = Dataset.from_dict({})

        self.dataset = defaultdict(
            lambda: defaultdict(
                lambda: RetrievalSplitData(
                    corpus=empty_dataset,
                    queries=empty_dataset,
                    relevant_docs={},
                    top_ranked=None,
                )
            )
        )

        def _process_split(
            ds_queries: dict | Dataset, ds_corpus: dict | Dataset
        ) -> tuple[Dataset, Dataset]:
            if isinstance(ds_queries, dict):
                queries = Dataset.from_list(
                    [{"id": k, "text": v} for k, v in ds_queries.items()]
                )
            elif isinstance(ds_queries, Dataset):
                queries = ds_queries
            else:
                raise ValueError(f"Can't convert queries of type {type(ds_queries)}")

            if isinstance(ds_corpus, dict):
                corpus = Dataset.from_list(
                    [
                        {
                            "id": k,
                            "text": v if isinstance(v, str) else v["text"],
                            "title": v.get("title", "") if isinstance(v, dict) else "",
                        }
                        for k, v in ds_corpus.items()
                    ]
                )
            elif isinstance(ds_corpus, Dataset):
                corpus = ds_corpus
            else:
                raise ValueError(f"Can't convert corpus of type {type(ds_corpus)}")
            return queries, corpus

        if self.metadata.is_multilingual:
            for subset in self.queries:  # type: ignore[attr-defined]
                for split in self.queries[subset]:  # type: ignore[attr-defined]
                    queries = self.queries[subset][split]  # type: ignore[attr-defined]
                    corpus = self.corpus[subset][split]  # type: ignore[attr-defined]

                    (
                        self.dataset[subset][split]["queries"],
                        self.dataset[subset][split]["corpus"],
                    ) = _process_split(queries, corpus)

                    self.dataset[subset][split]["relevant_docs"] = self.relevant_docs[  # type: ignore[attr-defined]
                        subset
                    ][split]
                    if hasattr(self, "instructions"):
                        instructions = self.instructions[subset][split]
                        self.dataset[subset][split]["queries"] = (
                            _combine_queries_with_instructions_datasets(
                                self.dataset[subset][split]["queries"],
                                instructions,
                                num_proc,
                            )
                        )
                    if hasattr(self, "top_ranked"):
                        self.dataset[subset][split]["top_ranked"] = self.top_ranked[
                            subset
                        ][split]
        else:
            subset = "default"
            for split in self.queries:  # type: ignore[attr-defined]
                queries = self.queries[split]  # type: ignore[attr-defined]
                corpus = self.corpus[split]  # type: ignore[attr-defined]
                (
                    self.dataset[subset][split]["queries"],
                    self.dataset[subset][split]["corpus"],
                ) = _process_split(queries, corpus)

                self.dataset[subset][split]["relevant_docs"] = self.relevant_docs[  # type: ignore[attr-defined]
                    split
                ].copy()
                if hasattr(self, "instructions"):
                    instructions = self.instructions[split]
                    self.dataset[subset][split]["queries"] = (
                        _combine_queries_with_instructions_datasets(
                            self.dataset[subset][split]["queries"],
                            instructions,
                            num_proc,
                        )
                    )
                if hasattr(self, "top_ranked") and self.top_ranked:
                    self.dataset[subset][split]["top_ranked"] = self.top_ranked[
                        split
                    ].copy()

        del self.queries  # type: ignore[attr-defined]
        del self.corpus  # type: ignore[attr-defined]
        del self.relevant_docs  # type: ignore[attr-defined]
        if hasattr(self, "instructions"):
            del self.instructions
        if hasattr(self, "top_ranked"):
            del self.top_ranked

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        """Load the dataset for the retrieval task."""
        if self.data_loaded:
            return

        dataset_path = self.metadata.dataset["path"]
        eval_splits = self.eval_splits
        trust_remote_code = self.metadata.dataset.get("trust_remote_code", False)
        revision = self.metadata.dataset["revision"]

        def _process_data(split: str, hf_subset: str = "default"):
            """Helper function to load and process data for a given split and language"""
            logger.debug(
                f"Loading {split} split for {hf_subset} subset of {self.metadata.name}"
            )

            self.dataset[hf_subset][split] = RetrievalDatasetLoader(
                hf_repo=dataset_path,
                revision=revision,
                trust_remote_code=trust_remote_code,
                split=split,
                config=hf_subset,
            ).load(
                num_proc=num_proc,
            )

        if self.metadata.is_multilingual:
            for lang in self.hf_subsets:
                for split in eval_splits:
                    _process_data(split, lang)
        else:
            for split in eval_splits:
                _process_data(split)
        self.dataset_transform(num_proc=num_proc)
        self.data_loaded = True

    def evaluate(
        self,
        model: MTEBModels,
        split: str = "test",
        subsets_to_run: list[HFSubset] | None = None,
        *,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        num_proc: int = 1,
        **kwargs: Any,
    ) -> Mapping[HFSubset, ScoresDict]:
        """Evaluate the model on the retrieval task.

        Args:
            model: Model to evaluate. Model should implement the [SearchProtocol][mteb.models.models_protocols.SearchProtocol]
                or be an [Encoder][mteb.models.models_protocols.EncoderProtocol] or [CrossEncoderProtocol][mteb.models.models_protocols.CrossEncoderProtocol].
            split: Split to evaluate on
            subsets_to_run: Optional list of subsets to evaluate on
            encode_kwargs: Keyword arguments passed to the encoder
            prediction_folder: Folder to save model predictions
            num_proc: Number of processes to use
            **kwargs: Additional keyword arguments passed to the evaluator

        Returns:
            Dictionary mapping subsets to their evaluation scores
        """
        if not self.data_loaded:
            self.load_data(num_proc=num_proc)
        # TODO: convert all tasks directly https://github.com/embeddings-benchmark/mteb/issues/2030
        self.convert_v1_dataset_format_to_v2(num_proc=num_proc)

        return super().evaluate(
            model,
            split,
            subsets_to_run,
            encode_kwargs=encode_kwargs,
            prediction_folder=prediction_folder,
            num_proc=num_proc,
            **kwargs,
        )

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: RetrievalSplitData,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int = 1,
        **kwargs,
    ) -> ScoresDict:
        """Evaluate a model on a specific subset of the data.

        Args:
            model: Model to evaluate
            data_split: Data split to evaluate on
            encode_kwargs: Keyword arguments passed to the encoder
            hf_split: Split to evaluate on
            hf_subset: Subset to evaluate on
            prediction_folder: Folder with results prediction
            num_proc: Number of processes to use
            **kwargs: Additional keyword arguments passed to the evaluator

        Returns:
            Dictionary of evaluation scores
        """
        # ensure queries format (see #3030)
        data_split["relevant_docs"], data_split["queries"] = (
            _filter_queries_without_positives(
                data_split["relevant_docs"], data_split["queries"]
            )
        )
        retriever = RetrievalEvaluator(
            corpus=data_split["corpus"],
            queries=data_split["queries"],
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            top_ranked=data_split["top_ranked"],
            top_k=self._top_k,
            **kwargs,
        )

        search_model: SearchProtocol

        if isinstance(model, EncoderProtocol) and not isinstance(model, SearchProtocol):
            search_model = SearchEncoderWrapper(model)
        elif isinstance(model, CrossEncoderProtocol):
            search_model = SearchCrossEncoderWrapper(model)
        elif isinstance(model, SearchProtocol):
            search_model = model
        else:
            raise TypeError(
                f"RetrievalEvaluator expects a SearchInterface, Encoder, or CrossEncoder, got {type(model)}"
            )

        start_time = time()
        results = retriever(
            search_model,
            encode_kwargs=encode_kwargs,
            num_proc=num_proc,
        )
        end_time = time()
        logger.debug(
            f"Running retrieval task - Time taken to retrieve: {end_time - start_time:.2f} seconds"
        )

        if prediction_folder:
            self._save_task_predictions(
                results,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        logger.info("Running retrieval task - Evaluating retrieval scores...")
        (
            all_scores,
            ndcg,
            _map,
            recall,
            precision,
            naucs,
            mrr,
            naucs_mrr,
            cv_recall,
        ) = retriever.evaluate(
            data_split["relevant_docs"],
            results,
            self.k_values,
            ignore_identical_ids=self.ignore_identical_ids,
            skip_first_result=self.skip_first_result,
        )
        task_specific_scores = self.task_specific_scores(
            all_scores,
            data_split["relevant_docs"],
            results,
            hf_split=hf_split,
            hf_subset=hf_subset,
        )
        logger.info("Running retrieval task - Finished.")
        return make_score_dict(
            ndcg,
            _map,
            recall,
            precision,
            mrr,
            naucs,
            naucs_mrr,
            cv_recall,
            task_specific_scores,
            self._previous_results_model_meta,
        )

    def task_specific_scores(
        self,
        scores: dict[str, dict[str, float]],
        qrels: RelevantDocumentsType,
        results: dict[str, dict[str, float]],
        hf_split: str,
        hf_subset: str,
    ) -> dict[str, float]:
        """Calculate task specific scores. Override in subclass if needed.

        Args:
            scores: Dictionary of scores
            qrels: Relevant documents
            results: Retrieval results
            hf_split: Split to evaluate on
            hf_subset: Subset to evaluate on
        """
        return {}

    def _calculate_descriptive_statistics_from_split(
        self,
        split: str,
        hf_subset: str | None = None,
        compute_overall: bool = False,
        num_proc: int = 1,
    ) -> RetrievalDescriptiveStatistics:
        self.convert_v1_dataset_format_to_v2(num_proc)
        if hf_subset and hf_subset in self.dataset:
            split_data = self.dataset[hf_subset][split]
            queries = split_data["queries"]
            corpus = split_data["corpus"]
            relevant_docs = split_data["relevant_docs"]
            top_ranked = split_data["top_ranked"]
        elif compute_overall:
            queries = None
            corpus = None
            relevant_docs = {}
            top_ranked = {}
            for hf_subset in self.metadata.eval_langs:
                split_data = self.dataset[hf_subset][split]
                if queries is None:
                    queries = split_data["queries"]
                else:
                    queries = concatenate_datasets([queries, split_data["queries"]])
                if corpus is None:
                    corpus = split_data["corpus"]
                else:
                    corpus = concatenate_datasets([corpus, split_data["corpus"]])

                relevant_docs.update(
                    _process_relevant_docs(
                        split_data["relevant_docs"], hf_subset, split
                    )
                )

                if "top_ranked" in split_data and split_data["top_ranked"] is not None:
                    top_ranked.update(
                        {
                            f"{split}_{hf_subset}_{k}": v
                            for k, v in split_data["top_ranked"].items()
                        }
                    )
        else:
            if "default" in self.dataset and split != "default":
                return self._calculate_descriptive_statistics_from_split(
                    split=split, hf_subset="default"
                )
            split_data = self.dataset["default"][split]
            queries = split_data["queries"]
            corpus = split_data["corpus"]
            relevant_docs = split_data["relevant_docs"]
            top_ranked = split_data["top_ranked"]

        num_documents = len(corpus)
        num_queries = len(queries)

        if self.metadata.category is None:
            queries_modalities = "t"
            corpus_modalities = "t"
        else:
            queries_modalities, corpus_modalities = self.metadata.category.split("2")

        number_of_characters = 0

        documents_text_statistics = None
        documents_image_statistics = None
        queries_text_statistics = None
        queries_image_statistics = None

        if "t" in corpus_modalities:
            corpus_texts = corpus.map(_corpus_to_dict)["text"]
            documents_text_statistics = calculate_text_statistics(corpus_texts)
            number_of_characters += documents_text_statistics["total_text_length"]

        if "i" in corpus_modalities:
            documents_image_statistics = calculate_image_statistics(corpus["image"])

        if "t" in queries_modalities:
            queries_ = queries
            if "instruction" in queries_[0]:
                queries_ = queries_.map(_combine_queries_with_instruction_text)

            if isinstance(queries_["text"][0], dict | list):
                queries_ = queries_.map(_convert_conv_history_to_query)
            queries_text_statistics = calculate_text_statistics(queries_["text"])

            number_of_characters += queries_text_statistics["total_text_length"]

        if "i" in queries_modalities:
            queries_image_statistics = calculate_image_statistics(queries["image"])

        relevant_docs_statistics = calculate_relevant_docs_statistics(relevant_docs)

        if top_ranked is not None and num_queries and len(top_ranked) > 0:
            top_ranked_statistics = calculate_top_ranked_statistics(
                top_ranked, num_queries
            )
        else:
            top_ranked_statistics = None

        return RetrievalDescriptiveStatistics(
            num_samples=num_documents + num_queries,
            number_of_characters=number_of_characters,
            documents_text_statistics=documents_text_statistics,
            documents_image_statistics=documents_image_statistics,
            queries_text_statistics=queries_text_statistics,
            queries_image_statistics=queries_image_statistics,
            relevant_docs_statistics=relevant_docs_statistics,
            top_ranked_statistics=top_ranked_statistics,
        )

    def _push_dataset_to_hub(self, repo_name: str, num_proc: int = 1) -> None:
        self.convert_v1_dataset_format_to_v2(num_proc)

        def _push_section(
            data: dict[str, RetrievalSplitData],
            subset_item: Literal["corpus", "queries", "relevant_docs", "top_ranked"],
            hf_subset_name: str,
            converter: Callable[[Any, Any], dict[str, Any]] | None = None,
        ) -> None:
            """Helper function to push dataset

            Args:
                data: Dataset with all items
                subset_item: Select which part to take. E. g. corpus, queries etc
                hf_subset_name: Name of the current item on HF
                converter: Function to convert dict to datasets format
            """
            sections = {}
            for split in data.keys():
                # skip empty instructions and top ranked
                if subset_item not in data[split] or data[split][subset_item] is None:
                    continue
                if isinstance(data[split][subset_item], Dataset):
                    sections[split] = data[split][subset_item]
                elif converter is not None:
                    subset_data = data[split][subset_item]
                    if subset_data is None:
                        continue

                    sections[split] = Dataset.from_list(
                        [converter(idx, item) for idx, item in subset_data.items()]
                    )
                else:
                    raise ValueError(
                        f"Unexpected subset item type {subset_item} without converter"
                    )
            if len(sections) > 0:
                DatasetDict(sections).push_to_hub(
                    repo_name,
                    hf_subset_name,
                    commit_message=f"Add {hf_subset_name}-{subset_item}",
                    num_proc=num_proc,
                )

        for subset in self.dataset:
            logger.info(f"Converting {subset} of {self.metadata.name}")
            _push_section(
                self.dataset[subset],
                "queries",
                f"{subset}-queries" if subset != "default" else "queries",
            )
            _push_section(
                self.dataset[subset],
                "corpus",
                f"{subset}-corpus" if subset != "default" else "corpus",
            )
            # Handle relevant_docs separately since one entry expands to multiple records.
            relevant_sections = {}
            for split, values in self.dataset[subset].items():
                relevant_docs = values["relevant_docs"]
                entries = []
                for query_id, docs in relevant_docs.items():
                    for doc_id, score in docs.items():
                        entries.append(
                            {
                                "query-id": query_id,
                                "corpus-id": doc_id,
                                "score": score,
                            }
                        )
                relevant_sections[split] = Dataset.from_list(entries)
            DatasetDict(relevant_sections).push_to_hub(
                repo_name,
                f"{subset}-qrels" if subset != "default" else "qrels",
                commit_message=f"Add {subset}-qrels",
                num_proc=num_proc,
            )

            _push_section(
                self.dataset[subset],
                "top_ranked",
                f"{subset}-top_ranked" if subset != "default" else "top_ranked",
                lambda idx, docs: {"query-id": idx, "corpus-ids": docs},
            )

    def convert_to_reranking(
        self,
        top_ranked_path: str | Path,
        top_k: int = 10,
    ) -> Self:
        """Converts a reranking task to re-ranking by loading predictions from previous model run where the `prediction_folder` was specified.

        Args:
            top_ranked_path: Path to file or folder with the top ranked predictions.
            top_k: Number of results to load.

        Returns:
            The current task reformulated as a reranking task

        Raises:
            FileNotFoundError: If the specified path does not exist.
            ValueError: If the loaded top ranked results are not in the expected format.
        """
        self._top_k = top_k

        top_ranked_path = Path(top_ranked_path)
        if top_ranked_path.is_dir():
            top_ranked_path = self._predictions_path(top_ranked_path)

        if not top_ranked_path.exists():
            raise FileNotFoundError(
                f"Can't find previous results for this task. File {top_ranked_path} does not exist."
            )

        with top_ranked_path.open("r") as previous_results_file:
            previous_results = json.load(previous_results_file)

        if not self.data_loaded:
            self.load_data()

        self._previous_results_model_meta = previous_results["mteb_model_meta"]

        for subset in self.dataset:
            for split in self.dataset[subset]:
                top_ranked: RetrievalOutputType = previous_results[subset][split]
                if not isinstance(top_ranked, dict):
                    raise ValueError("Previous top ranked results is not a dictionary.")

                top_k_sorted = defaultdict(list)
                for query_id, values in top_ranked.items():
                    sorted_keys = sorted(values, key=lambda k: values[k], reverse=True)
                    top_k_sorted[query_id] = sorted_keys[: self._top_k]

                self.dataset[subset][split]["top_ranked"] = top_k_sorted
        return self


def _process_relevant_docs(
    collection: Mapping[str, Mapping[str, int]],
    hf_subset: str,
    split: str,
) -> dict[str, dict[str, int]]:
    """Collections can contain overlapping ids in different splits. Prepend split and subset to avoid this

    Returns:
        A new collection with split and subset prepended to ids
    """
    return_collection = {}
    for query_id, relevant in collection.items():
        return_collection[f"{split}_{hf_subset}_{query_id}"] = {
            f"{split}_{hf_subset}_{doc_id}": value for doc_id, value in relevant.items()
        }
    return return_collection



import logging

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

logger = logging.getLogger(__name__)

_EVAL_SPLIT = "test"
_LANGUAGES = {
    "us": ["eng-Latn"],
    "es": ["spa-Latn"],
    "jp": ["jpn-Jpan"],
}

_CITATION = r"""@article{reddy2022shopping,
  archiveprefix = {arXiv},
  author = {Chandan K. Reddy and Lluís Màrquez and Fran Valero and Nikhil Rao and Hugo Zaragoza and Sambaran Bandyopadhyay and Arnab Biswas and Anlu Xing and Karthik Subbian},
  eprint = {2206.06588},
  title = {Shopping Queries Dataset: A Large-Scale {ESCI} Benchmark for Improving Product Search},
  year = {2022},
}"""


class ESCIReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ESCIReranking",
        description="",
        reference="https://github.com/amazon-science/esci-data/",
        dataset={
            "path": "mteb/ESCIReranking",
            "revision": "dc2cfaf4fcbf238806a02ae8607786e88112463e",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=_LANGUAGES,
        main_score="map_at_1000",
        date=("2022-06-14", "2022-06-14"),
        domains=["Written"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation=_CITATION,
    )



from collections import defaultdict

from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class NanoClimateFeverRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NanoClimateFeverRetrieval",
        description="NanoClimateFever is a small version of the BEIR dataset adopting the FEVER methodology that consists of 1,535 real-world claims regarding climate-change.",
        reference="https://arxiv.org/abs/2012.00614",
        dataset={
            "path": "zeta-alpha-ai/NanoClimateFEVER",
            "revision": "96741bfa30b9f56db8c9eb7d08e775ed6474f206",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=["2020-01-01", "2020-12-31"],
        domains=["Non-fiction", "Academic", "News"],
        task_subtypes=["Claim verification"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{diggelmann2021climatefever,
  archiveprefix = {arXiv},
  author = {Thomas Diggelmann and Jordan Boyd-Graber and Jannis Bulian and Massimiliano Ciaramita and Markus Leippold},
  eprint = {2012.00614},
  primaryclass = {cs.CL},
  title = {CLIMATE-FEVER: A Dataset for Verification of Real-World Climate Claims},
  year = {2021},
}
""",
        prompt={
            "query": "Given a claim about climate change, retrieve documents that support or refute the claim"
        },
        adapted_from=["ClimateFEVER"],
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus = load_dataset(
            "zeta-alpha-ai/NanoClimateFEVER",
            "corpus",
            revision="96741bfa30b9f56db8c9eb7d08e775ed6474f206",
        )
        self.queries = load_dataset(
            "zeta-alpha-ai/NanoClimateFEVER",
            "queries",
            revision="96741bfa30b9f56db8c9eb7d08e775ed6474f206",
        )
        self.relevant_docs = load_dataset(
            "zeta-alpha-ai/NanoClimateFEVER",
            "qrels",
            revision="96741bfa30b9f56db8c9eb7d08e775ed6474f206",
        )

        self.corpus = {
            split: {
                sample["_id"]: {"_id": sample["_id"], "text": sample["text"]}
                for sample in self.corpus[split]
            }
            for split in self.corpus
        }

        self.queries = {
            split: {sample["_id"]: sample["text"] for sample in self.queries[split]}
            for split in self.queries
        }

        relevant_docs = {}

        for split in self.relevant_docs:
            relevant_docs[split] = defaultdict(dict)
            for query_id, corpus_id in zip(
                self.relevant_docs[split]["query-id"],
                self.relevant_docs[split]["corpus-id"],
            ):
                relevant_docs[split][query_id][corpus_id] = 1
        self.relevant_docs = relevant_docs

        self.data_loaded = True
